/**
 * \file gre/elemparms.h
 * \brief GRE element parameter definitions.
 *
 * \if NODOC
 * $Id: elemparms.h_v 1.8 2012/05/25 17:14:11 mju Exp $
 *
 * $Log: elemparms.h_v $
 * Revision 1.8  2012/05/25 17:14:11  mju
 * Move autolabelparms to own hdr.
 *
 * Revision 1.7  2011/10/20 12:53:54  mju
 * Remove redundant hdrs.
 *
 * Revision 1.6  2009/02/26 13:31:12  mju
 * Use mistring for styleparms.imageFieldName.
 *
 * Revision 1.5  2007/04/26 22:56:12  bcolombe
 * Added ImageFieldName to STYLEPARMS
 *
 * Revision 1.4  2006/09/28 13:14:57  mju
 * Use legendparms hdr.
 *
 * Revision 1.3  2006/09/28 13:04:04  mju
 * Use datatip hdr.
 *
 * Revision 1.2  2006/09/27 16:17:39  mju
 * Move extruded parm classes to own hdr.
 *
 * Revision 1.1  2006/09/21 22:00:32  mju
 * Initial revision
 *
 * \endif
**/

#ifndef	INC_GRE_ELEMPARMS_H
#define	INC_GRE_ELEMPARMS_H

#ifndef	INC_GRE_BASE_H
	#include <gre/base.h>
#endif

#ifndef	INC_MI32_GREMODE_H
	#include <mi32/gremode.h>
#endif

#ifndef	INC_MI32_LABLSTYL_H
	#include <mi32/lablstyl.h>
#endif

#ifndef	INC_GRE_DATATIP_H
	#include <gre/datatip.h>
#endif

#ifndef	INC_GRE_LEGENDPARMS_H
	#include <gre/legendparms.h>
#endif

#ifndef	INC_GRE_AUTOLABELPARMS_H
	#include <gre/autolabelparms.h>
#endif



namespace GRE {
//===================================================================================================================
//		Parameters needed for various element types

//! Selection parameters for determining which elements to draw
struct SELECTPARMS {
	public:
		UINT16 Mode;									//!< Selection mode - SELECTMODE_...
		INT16 Table;									//!< 'Selection' table number
		MIUNICODE *QueryStr;							//!< Query script for selection by query
		RVC::OBJECTNAME TableName;					//!< 'Selection' table name

		SELECTPARMS (									//! Default constructor
			);
		SELECTPARMS (									//! Copy constructor
			const SELECTPARMS& rhs
			);
		~SELECTPARMS ();								//! Destructor

		SELECTPARMS& operator= (					//! Assignment
			const SELECTPARMS& rhs
			);

		//! Check and validate for specified database and optionally fix mode if none valid.
		void CheckTable (
			const RVC::DBASE& db,					//!< Database object
			bool fixmode								//!< Fix selection mode if necessary
			);

		//! Determine if need to have a valid database table.
		bool NeedTable (
			) const { return ((Mode & SELECTMODE_NeedsTable) != 0); }

		//! Validate settings and fix if necessary.
		void Validate ();
	};

//! Basic style parameters for element drawing
struct STYLEPARMS {
	public:
		UINT16 Mode;									//!< Style mode - STYLEMODE_...
		INT16 Table;									//!< 'Style assignment' table number
		MIUNICODE *QueryStr;							//!< Query string for styling by scipt
		RVC::OBJECTNAME TableName;					//!< 'Style Assignment' table name
		MISTRING m_ImageFieldName;					//!< FieldName for Image Thumbnail

		STYLEPARMS (									//! Default constructor
			);
		STYLEPARMS (									//! Copy constructor
			const STYLEPARMS& rhs
			);
		~STYLEPARMS ();								//! Destructor

		STYLEPARMS& operator= (						//! Assignment
			const STYLEPARMS& rhs
			);

		//! Check and validate for specified database and optionally fix mode if none valid.
		void CheckTable (
			const RVC::DBASE& db,					//!< Database object
			bool fixmode,								//!< Fix selection mode if necessary
			const RVC::STYLETYPE styletype = RVC::STYLETYPE_NumTypes
			);

		//! Determine if need to have a valid database table.
		bool NeedTable (
			) const { return ((Mode & STYLEMODE_NeedsTable) != 0); }

		//! Validate settings and fix if necessary.
		void Validate ();
	};

//! Point/Node element parameters.
struct POINTPARMS {
	public:

		struct POINTSTYLEPARMS : public STYLEPARMS {
			POINTSTYLE Normal;						//!< Style to use in normal drawing
			};

		SELECTPARMS Select;
		POINTSTYLEPARMS Style;
		DATATIPPARMS DataTip;						//!< DataTip parameters
		AUTOLABELPARMS AutoLabel;
		LEGENDPARMS Legend;
		RVC::OBJECTNAME LegendElemTableName;

		POINTPARMS ();
		POINTPARMS (const POINTPARMS& rhs);
		~POINTPARMS ();

		POINTPARMS& operator= (const POINTPARMS& rhs);
		void Validate ();
	};

//! Line element parameters
struct LINEPARMS {
	public:

		struct LINESTYLEPARMS : public STYLEPARMS {
			LINESTYLE Normal;					//!< Style to use in normal drawing
			};

		SELECTPARMS Select;
		LINESTYLEPARMS Style;
		DATATIPPARMS DataTip;				//!< DataTip parameters
		AUTOLABELPARMS AutoLabel;
		LEGENDPARMS Legend;
		RVC::OBJECTNAME LegendElemTableName;

		LINEPARMS ();
		LINEPARMS (const LINEPARMS& rhs);
		~LINEPARMS ();

		LINEPARMS& operator= (const LINEPARMS& rhs);
		void Validate ();
	};

//! Polygon/Area element parameters
struct POLYPARMS {
	public:

		struct POLYSTYLEPARMS : public STYLEPARMS {
			POLYSTYLE Normal;					//!< Style to use in normal drawing
			};

		SELECTPARMS Select;
		POLYSTYLEPARMS Style;
		DATATIPPARMS DataTip;				//!< DataTip parameters
		AUTOLABELPARMS AutoLabel;
		LEGENDPARMS Legend;
		LEGENDPARMS LegendInt;				//!< Label inside the polygon sample
		RVC::OBJECTNAME LegendElemTableName;
		bool ShowInteriorLabel;				//!< Show the label inside the polygon sample
		bool ShowLabel;						//!< Show the label outside the polygon sample

		POLYPARMS ();
		POLYPARMS (const POLYPARMS& rhs);
		~POLYPARMS ();

		POLYPARMS& operator= (const POLYPARMS& rhs);
		void Validate ();
	};

//! Label/Text element parameters
struct LABELPARMS {
	public:

		struct LABELSTYLEPARMS : public STYLEPARMS {
			TEXTSTYLE Normal;					//!< Style to use in normal drawing
			LABELFRAMEPARMS FrameParms;
			
			LABELSTYLEPARMS ();
			};

		SELECTPARMS Select;
		LABELSTYLEPARMS Style;
		DATATIPPARMS DataTip;				//!< DataTip parameters
		double MinScaleVisible;				//!< Minimum scale at which labels are visible
		double MaxScaleVisible;				//!< Maximum scale at which labels are visible

		LABELPARMS ();
		LABELPARMS (const LABELPARMS& rhs);
		~LABELPARMS ();

		LABELPARMS& operator= (const LABELPARMS& rhs);

		//! Determine if labels are visible at specified scale.
		bool IsVisibleAtScale (
			double scale						//!< Map scale
			) const { return (scale >= MinScaleVisible && (MaxScaleVisible <= 0.0 || scale <= MaxScaleVisible)); }
		void Validate ();
	};

//===================================================================================================================
}	// End namespace GRE
#endif	// INC_GRE_ELEMPARMS_H
