/** 
 * \file mgui/ctrl.h
 * \brief Definitions for MGUI::CTRL classes.
 *
 * \if NODOC
 * $Id: ctrl.h_v 1.118 2011/11/16 15:23:43 mju Exp $
 *
 * $Log: ctrl.h_v $
 * Revision 1.118  2011/11/16 15:23:43  mju
 * dox tweaks.
 *
 * Revision 1.117  2011/11/11 14:49:51  mju
 * Remove mfc stuff.
 *
 * Revision 1.114  2011/08/24 19:21:28  mju
 * Add pushbutton::style_TextAtLeft.
 *
 * Revision 1.113  2011/08/24 15:09:18  mju
 * Make ctrl::getWidget const.
 *
 * Revision 1.112  2011/08/22 17:13:31  mju
 * Add pushbutton::updateIcon.
 *
 * Revision 1.111  2011/08/19 20:33:12  mju
 * Add ctrl::setWidgetShadow/setShadowArgs and shadow enum.
 * Add pushbutton styles for highlight/shadow on mouseover and new create method to use.
 * Deprecate pushbutton::create that uses bool for setting "large" button.
 *
 * Revision 1.109  2010/03/12 21:41:21  mju
 * Add invalidate for X.
 *
 * Revision 1.108  2010/03/08 23:25:05  mju
 * Add pushbutton::style_SetCursor.
 *
 * Revision 1.107  2010/03/08 17:15:02  mju
 * Add sizealign for pushbutton::create.
 *
 * Revision 1.106  2010/03/08 16:35:03  mju
 * Add event handlers for mouseover icon.
 *
 * Revision 1.105  2010/03/08 15:51:24  mju
 * Add pixmaps for mouseover and style for pushbutton.
 *
 * Revision 1.104  2010/03/05 22:51:50  mju
 * Allow use of mgui::icon.
 *
 * Revision 1.101  2009/09/10 21:01:46  mju
 * Add groupbox::boxstyle and use when create with no label.
 *
 * Revision 1.99  2009/02/04 23:08:49  bcolombe
 * Added SetForegroundColor
 *
 * Revision 1.96  2008/04/11 16:15:20  mju
 * Add getShell.
 *
 * Revision 1.94  2008/02/22 22:07:02  mju
 * Add get/setDimensions.
 *
 * Revision 1.93  2007/11/29 20:52:58  scowan
 * Moved keystate enum to its own include file.
 *
 * Revision 1.92  2007/11/09 18:33:51  mju
 * Move statusbar to own hdr.
 * Add ctrl::processWidgetEvents.
 *
 * Revision 1.86  2007/07/12 17:28:57  mju
 * Add get/setBackgroundColor.
 *
 * Revision 1.76  2006/07/21 20:49:14  mju
 * Add ctrl.getToolTip and setToolTipString methods.
 *
 * Revision 1.75  2006/07/20 15:18:13  mju
 * Add setMinimumSize.
 *
 * Revision 1.73  2006/05/01 17:52:46  scowan
 * Added bitmap member to label.
 *
 * Revision 1.71  2006/03/27 17:29:00  scowan
 * Added a setIcon method for label.
 *
 * Revision 1.70  2006/03/14 18:00:22  mju
 * Add setSizeByKey.
 *
 * Revision 1.59  2005/03/11 18:56:54  mju
 * Add value member for togglebutton so retains state after control destroyed.
 *
 * Revision 1.54  2004/07/27 13:42:09  mju
 * Add groupbox.setlabeltext.
 *
 * Revision 1.48  2004/03/15 17:45:57  dwilliss
 * Added a SetMaxLength() method to edit controls
 *
 * Revision 1.46  2003/12/12 18:49:36  mju
 * Move 'selection' methods to edit_base.
 * Add set/clear selection methods to ctrl_text.
 *
 * Revision 1.1  2001/12/03 17:38:38  mju
 * Initial revision
 * \endif
**/

#ifndef INC_MGUI_CTRL_H
#define	INC_MGUI_CTRL_H

#ifndef INC_MGUI_FORM_H
#include <mgui/form.h>
#endif

#ifndef INC_MGUI_KEYSTATE_H
#include <mgui/keystate.h>
#endif

#ifndef	INC_MI32_RECT_H
#include <mi32/rect.h>
#endif

#ifndef	INC_MI32_ICONID_H
#include <mi32/iconid.h>
#endif

#ifndef INC_MI32_XBASE_H
#include <mi32/xbase.h>
#endif


#ifndef GENERATING_DOXYGEN_OUTPUT
// Forward declarations.
struct COLOR;
namespace MGUI {
	class CTRL_TOOLBAR;
	class CTRL_TOOLTIP;
	class SHELL;
	class ICON;
	}
#endif


namespace MGUI {

//===================================================================================================================
//! Base for "single" graphical user interface controls.
//!
//! There are two basic types of 'controls' which can be used in user interface implementation,
//! controls which the user can interact with and 'static' controls which the user cannot change
//! directly.
//!
//! All controls which the user can manipulate also encapsulate information about the current
//! 'state' of the control (except for MGUI::CTRL_PUSHBUTTON).  There are three common ways to
//! use each of these controls in a program (in order of convenience):
//!
//!		-# If changing the state of a control has no other immediate effect then the control class
//!			may be used directly and the state information retrieved when needed, such as when the
//!			user presses 'OK' on the dialog.
//!		-# Use the various SetDelegate... methods provided by the control to specify a method to
//!			be called when the user interacts with the control.
//!		-# Inherit from the control class and override the On... virtual method to perform task
//!			specific processing.  Note that not all controls allow such overrides.  Also, delegates
//!			should not be set when overriding as the order of calling delegate vs override is
//!			indeterminant and in some cases the delegate may not be called at all.
//!
class CTRL : public MGUI::FORM {
	public:

		enum SHADOW {
			SHADOW_None =		0,
			SHADOW_Black,
			SHADOW_White,
			SHADOW_In,
			SHADOW_Out
			};

		//! Delegate type for size change notification.
		//! Parameters are xsize, ysize.
		typedef fastdelegate::FastDelegate<void(int, int)> DELEGATE_ONSIZE;

		//! Get common interface 'background' color.
		static const COLOR& GetBackgroundColor (
			);

		//! Process all pending events for widget, discarding others.
		//! Intended for use in code which checks for a "button press" during
		//! an operation, ignoring user actions on other controls.
		static void ProcessWidgetEvents (
			Widget widget
			);

		//! Set shadow for widget.
		static void SetWidgetShadow (
			Widget widget,
			SHADOW shadow
			);

		//! Destructor.
		virtual ~CTRL (
			) = 0;

		//! Get control 'background' color.
		void GetBackgroundColor (
			COLOR& color
			) const;

		//! Get control dimensions.
		void GetDimensions (
			MGUI::POINT& dimensions		//!< Dimensions returned
			) const;

		//! Get parent for dialogs launched from control.
		MDLGPARENT GetDlgParent (
			) const;

		//! Get pointer to shell containing control.
		//! @return NULL if not contained in MGUI::SHELL.
		SHELL* GetShell (
			) const;

		//! Get associated ToolTip control.
		//! @return NULL if no ToolTip control attached
		CTRL_TOOLTIP* GetToolTip (
			) const;

		//! Get the Widget for the control or container if multiple controls (X only).
		//! The widget should only be used for attachment, do not assume the
		//! widget is of a particular type.
		Widget GetWidget (
			) const { return (m_widget); }

		//! Invalidate the entire control area.
		//! This will force it to redraw during normal event processing.
		void Invalidate (
			bool erase = true
			);

		//! Determine if control has been created.
		//! @return true if created, false if not.
		virtual bool IsCreated (
			) const;

		//! Determine if control is enabled for keyboard or mouse input.
		//! @return true if enabled, false if not, or if not yet created.
		virtual bool IsEnabled (
			) const;

		//! Determine if control is visible.
		//! @return true if visible, false if not, or if not yet created.
		bool IsVisible (
			) const;

		//! Set control 'background' color.
		void SetBackgroundColor (
			const COLOR& color
			);

		//! Set control dimensions if allowed.
		void SetDimensions (
			MGUI::POINT dimensions		//!< New dimensions requested
			);

		//! Set whether mouse or keyboard input to control is allowed.
		virtual void SetEnabled (
			bool enabled = true
			);

		//! Set control 'foreground' color.
		void SetForegroundColor (
			const COLOR& color
			);

		//! Set minimum size for control.
		void SetMinimumSize (
			int width,						//!< Minimum width in pixels, -1 to leave unchanged
			int height						//!< Minimum height in pixels, -1 to leave unchanged
			);

		//! Set control size using key.
		//! This will automatically load the size and subsequently
		//! record any changes later made to the size by the user.
		void SetSizeByKey (
			const char *SizeKey
			);

		//! Set ToolTip string.
		//! Recommended for simple ToolTip support only.
		//! For complex or dynamic ToolTips use the CTRL_TOOLTIP class directly.
		void SetToolTipString (
			const MISTRING& string			//!< String to use for ToolTip
			);

		//! Set whether control is visible or not.
		virtual void SetVisible (
			bool visible = true
			);

		//! Force window to be updated immediately if necessary.
		void UpdateWindow (
			);

	protected:

		// Set shadow values in XTARGS.
		static void SetShadowArgs (XTARGS& args, SHADOW shadow);

		Widget m_widget;

		//! Constructor (X version).
		CTRL (
			): m_widget(0) { }

		//! Add 'Destroy' callback for widget.
		void AddDestroyCallback (
			);

		//! Called if widgets are destroyed before CTRL class instance.
		//! Will not be called if CTRL class instance is destroyed before widgets.
		//! Usually just sets widgets to NULL so won't be referenced in destructor.
		//! Derived class must call corresponding base class method AFTER performing its own processing.
		virtual void v_OnDestroyWidget (
			);

	private:
		#ifndef GENERATING_DOXYGEN_OUTPUT

		static void CB_Destroy (Widget, CTRL*, void*);

		// MGUI::FORM overrides.
		virtual void v_SetFocus () const;

		CTRL (const CTRL&);
		CTRL& operator= (const CTRL&);
		#endif // GENERATING_DOXYGEN_OUTPUT
	};


//===================================================================================================================
//! Non-editable label control.
class CTRL_LABEL : public MGUI::CTRL {
	public:

		enum STYLE {
			STYLE_LeftNoWrap,								//!< Left-justify text without word wrapping
			STYLE_Left,										//!< Left-justify, may word-wrap if too long
			STYLE_Center,									//!< Center, may word-wrap if too long
			STYLE_Right										//!< Right-justify, may word-wrap if too long
			};

		//! Constructor.
		CTRL_LABEL ();

		~CTRL_LABEL ();

		//! Create control with text.
		void Create (
			MGUI::LAYOUT_PANE_BASE& ParentPane,		//!< Parent pane
			const MISTRING& label,						//!< Label string (allows TEXTID)
			MGUI::LAYOUT_SIZEALIGN sizealign = MGUI::LAYOUT_SIZEALIGN_FixedSize,
			STYLE style = STYLE_LeftNoWrap
			);

		//! Create control with icon.
		void Create (
			MGUI::LAYOUT_PANE_BASE& ParentPane,		//!< Parent pane
			const MGUI::ICON& icon,						//!< Icon
			const MISTRING& tooltip						//!< Tooltip string (allows TEXTID)
			);

		//! Create control with ICONID with MISTRING or TEXTID tooltip
		void Create (
			MGUI::LAYOUT_PANE_BASE& ParentPane,		//!< Parent pane
			ICONID iconid,									//!< ICON to use as the label
			const MISTRING& tooltip,					//!< tooltip string (allows TEXTID)
			UINT16 iconsize = 0
			);

		//! Set new icon.
		void SetIcon (
			const MGUI::ICON& icon						//!< Icon to show
			);

		//! Set new icon.
		void SetIcon (
			ICONID iconid,									//!< ICONID to show
			UINT16 iconsize = 0
			);

		//! Set new label from Unicode string.
		void SetLabel (
			const MISTRING& string
			);

	private:
		#ifndef GENERATING_DOXYGEN_OUTPUT

		Pixmap m_PixmapNormal;
		Pixmap m_PixmapDisabled;

		void CreateIconX (
			MGUI::LAYOUT_PANE_BASE& ParentPane,
			const MGUI::ICON& icon
			);

		void CreateX (
			MGUI::LAYOUT_PANE_BASE& ParentPane,
			XTARGS& args,
			const char *name,
			MGUI::LAYOUT_SIZEALIGN sizealign,
			STYLE style
			);

		CTRL_LABEL (const CTRL_LABEL&);
		CTRL_LABEL& operator= (const CTRL_LABEL&);
		#endif // GENERATING_DOXYGEN_OUTPUT
	};


//===================================================================================================================
//!	Group Box control, used to show frame around other controls.
class CTRL_GROUPBOX : public MGUI::CTRL {
	public:

		enum BOXSTYLE {
			BOXSTYLE_EtchedIn,
			BOXSTYLE_EtchedOut,
			BOXSTYLE_ShadowIn,
			BOXSTYLE_ShadowOut
			};

		//! Constructor.
		CTRL_GROUPBOX (
			);

		//! Destructor.
		virtual ~CTRL_GROUPBOX (
			);

		//! Cast to MGUI::LAYOUT_PANE_BASE& for use as parent to other controls.
		operator MGUI::LAYOUT_PANE_BASE& (
			) { return (m_GroupBoxPane); }

		//! Create control with no label.
		void Create (
			MGUI::LAYOUT_PANE_BASE& ParentPane,	//!< Parent pane
			MGUI::LAYOUT_ORIENTATION orientation = MGUI::LAYOUT_ORIENTATION_Vertical,
			MGUI::LAYOUT_SIZEALIGN sizealign = MGUI::LAYOUT_SIZEALIGN_FixedSize,
			int ChildSpacing = 4,					//!< Spacing between children of group box
			int ExtraBorder = 0,						//!< Extra border around inside of group box, in addition to ChildSpacing
			BOXSTYLE BoxStyle = BOXSTYLE_EtchedIn
			);

		//! Create control with label.
		void Create (
			MGUI::LAYOUT_PANE_BASE& ParentPane,	//!< Parent pane
			const MISTRING& string,					//!< Label string
			MGUI::LAYOUT_ORIENTATION orientation = MGUI::LAYOUT_ORIENTATION_Vertical,
			MGUI::LAYOUT_SIZEALIGN sizealign = MGUI::LAYOUT_SIZEALIGN_FixedSize,
			int ChildSpacing = 4,					//!< Spacing between children of group box
			int ExtraBorder = 0						//!< Extra border around inside of group box, in addition to ChildSpacing
			);

		//! Get layout pane to invoke options on it.
		MGUI::LAYOUT_PANE_BASE& GetPane (
			) { return (m_GroupBoxPane); }

		void SetLabelText (
			const MISTRING& label
			);

	private:

		#ifndef GENERATING_DOXYGEN_OUTPUT
		class LAYOUT_PANE : public MGUI::LAYOUT_PANE_BASE {
			public:
				LAYOUT_PANE (
					) { }
				virtual ~LAYOUT_PANE (
					) { }
				void Create (
					Widget FrameWidget,
					MGUI::LAYOUT_ORIENTATION orientation,
					MGUI::LAYOUT_SIZEALIGN sizealign,
					int ChildSpacing,
					int ExtraBorder,
					int TopExtra
					);
			private:
				LAYOUT_PANE (const LAYOUT_PANE&);
				LAYOUT_PANE& operator= (const LAYOUT_PANE&);
			};

		LAYOUT_PANE m_GroupBoxPane;

		Widget m_label;
		void CreateX (
			MGUI::LAYOUT_PANE_BASE& ParentPane,
			MGUI::LAYOUT_ORIENTATION orientation,
			MGUI::LAYOUT_SIZEALIGN sizealign,
			int ChildSpacing,
			int ExtraBorder,
			BOXSTYLE BoxStyle = BOXSTYLE_EtchedIn
			);

		CTRL_GROUPBOX (const CTRL_GROUPBOX&);
		CTRL_GROUPBOX& operator= (const CTRL_GROUPBOX&);
		#endif // GENERATING_DOXYGEN_OUTPUT
	};


//===================================================================================================================
//! PushButton control supporting both text and icon buttons.
class CTRL_PUSHBUTTON : public MGUI::CTRL {
	public:

		//! Button styles.
		enum STYLE {
			STYLE_Default =					0,
			STYLE_NoFrame =					0x01,		//!< Don't include frame around button, cannot combine with STYLE_Shadow/HighlightOnMouseOver
			STYLE_NoMargin =					0x02,		//!< Don't include margin, typically only used for icon button
			STYLE_SetCursor =					0x04,		//!< Set standard cursor, for use when button in sash
			STYLE_WideMargins =				0x08,		//!< Use wider margins for text buttons, typically used for buttons at bottom of window
			STYLE_HighlightOnMouseOver =	0x10,		//!< Only show highlight frame on mouseover, cannot combine with STYLE_NoFrame or STYLE_ShadowOnMouseOver
			STYLE_ShadowOnMouseOver =		0x20,		//!< Only show shadow frame on mouseover, cannot combine with STYLE_NoFrame or STYLE_HighlightOnMouseOver
			STYLE_TextAtLeft =				0x40,		//!< Place text at left of button, for use when button expanded to fill width of pane
			};

		//! Constructor.
		CTRL_PUSHBUTTON ();

		//! Destructor.
		virtual ~CTRL_PUSHBUTTON ();

		//! Create control with text label.
		void Create (
			MGUI::LAYOUT_PANE_BASE& ParentPane,		//!< Parent pane
			const MISTRING& label,						//!< Label string
			STYLE style = STYLE_Default,				//!< Button style
			MGUI::LAYOUT_SIZEALIGN sizealign = MGUI::LAYOUT_SIZEALIGN_FixedSize
			);

		//! Create control with icon.
		void Create (
			MGUI::LAYOUT_PANE_BASE& ParentPane,		//!< Parent pane
			const MGUI::ICON& icon,						//!< Icon
			const MISTRING& tooltip,					//!< ToolTip string
			STYLE style = STYLE_Default,				//!< Button style
			MGUI::LAYOUT_SIZEALIGN sizealign = MGUI::LAYOUT_SIZEALIGN_FixedSize
			);

		//! Create control with different icons for "normal" and "mouse-over" states.
		void Create (
			MGUI::LAYOUT_PANE_BASE& ParentPane,		//!< Parent pane
			const MGUI::ICON& iconNormal,				//!< Icon
			const MGUI::ICON& iconMouseOver,			//!< Icon when mouse is over and not pressed
			const MISTRING& tooltip,					//!< ToolTip string
			STYLE style = STYLE_Default,				//!< Button style
			MGUI::LAYOUT_SIZEALIGN sizealign = MGUI::LAYOUT_SIZEALIGN_FixedSize
			);

		//! Create control with ICONID.
		void Create (
			MGUI::LAYOUT_PANE_BASE& ParentPane,		//!< Parent pane
			ICONID iconid,									//!< Icon ID
			const MISTRING& tooltip,					//!< ToolTip string
			UINT16 iconsize = 0,							//!< Icon size, 0 for default
			STYLE style = STYLE_Default,				//!< Button style
			MGUI::LAYOUT_SIZEALIGN sizealign = MGUI::LAYOUT_SIZEALIGN_FixedSize
			);

		//! Set delegate to call when button pressed.
		void SetDelegateOnPressed (
			DELEGATE_VOID_NOPARMS delegate
			) { m_DelegateOnPressed = delegate; }

		//! Update icon for button.
		//! Button must have been created with icon and new icon must match size of original.
		void UpdateIcon (
			const MGUI::ICON& icon						//!< Icon
			);

	private:
		#ifndef GENERATING_DOXYGEN_OUTPUT

		DELEGATE_VOID_NOPARMS m_DelegateOnPressed;

		static void CB_PushButton (Widget, CTRL_PUSHBUTTON*, void*);
		static void XEH_Icon (Widget, CTRL_PUSHBUTTON*, XEvent*, void*);
		static void XEH_Text (Widget, CTRL_PUSHBUTTON*, XEvent*, void*);

		Pixmap m_PixmapUnarmed;
		Pixmap m_PixmapDisabled;
		Pixmap m_PixmapArmed;
		Pixmap m_PixmapMouseOver;
		STYLE m_style;

		void CreateIconX (
			LAYOUT_PANE_BASE& ParentPane,
			const MGUI::ICON& Icon,
			const MGUI::ICON *pIconMouseOver,
			const MISTRING& tooltip,
			STYLE style,
			MGUI::LAYOUT_SIZEALIGN sizealign
			);

		void CreateWidget (
			LAYOUT_PANE_BASE& ParentPane,
			XTARGS& args,
			const char *name
			);

		CTRL_PUSHBUTTON (const CTRL_PUSHBUTTON&);
		CTRL_PUSHBUTTON& operator= (const CTRL_PUSHBUTTON&);
		#endif // GENERATING_DOXYGEN_OUTPUT

		// Overridables.

		//! Called when button is pushed.
		//! Default implementation calls delegate if has been set.
		virtual void OnPressed ();

	};
#ifndef GENERATING_DOXYGEN_OUTPUT
DEFINE_ENUM_OP_BITWISE(CTRL_PUSHBUTTON::STYLE)
#endif


//===================================================================================================================
//! Toggle button control supporting both "checkbox" and "radio" behavior with text and icon buttons.
//! Note that radio behavior -between- buttons is NOT enforced by this control.  For
//! portability it is recommended that the MGUI::FORM_RADIOBUTTONS class be used to manage a set of
//! radio buttons or that the code enforce radio behavior itself for more complex cases.
class CTRL_TOGGLEBUTTON : public MGUI::CTRL {
	public:

		enum STYLE {
			STYLE_Check =			0x00,					//!< Checkbox style
			STYLE_Radio =			0x01,					//!< Radiobutton style
			STYLE_NoMargin =		0x04,					//!< Don't include margins for icon
			STYLE_NoFrame =		0x08,					//!< Don't include frame for icon
			};

		//! Constructor.
		CTRL_TOGGLEBUTTON ();

		//! Destructor.
		virtual ~CTRL_TOGGLEBUTTON ();

		//! Create with text label.
		void Create (
			MGUI::LAYOUT_PANE_BASE& ParentPane,		//!< Parent pane
			const MISTRING& label,						//!< Label string
			STYLE style = STYLE_Check,					//!< Toggle style
			bool dftvalue = false,						//!< Default value
			MGUI::LAYOUT_SIZEALIGN sizealign = MGUI::LAYOUT_SIZEALIGN_FixedSize
			);

		//! Create with icon.
		void Create (
			MGUI::LAYOUT_PANE_BASE& ParentPane,		//!< Parent pane
			const MGUI::ICON& icon,						//!< Icon
			const MISTRING& tooltip,					//!< ToolTip string
			STYLE style = STYLE_Check,					//!< Toggle style
			bool dftvalue = false						//!< Default value
			);

		//! Create with different icons for "off" and "on" states.
		void Create (
			MGUI::LAYOUT_PANE_BASE& ParentPane,		//!< Parent pane
			const MGUI::ICON& IconOff,					//!< Icon for "off" state, will use "armed" state for mouseover when "on"
			const MGUI::ICON& IconOn,					//!< Icon for "on" state, will use "unarmed" state for mouseover when "off"
			const MISTRING& tooltip,					//!< ToolTip string
			STYLE style = STYLE_Check,					//!< Toggle style
			bool dftvalue = false						//!< Default value
			);

		//! Create with ICONID.
		void Create (
			MGUI::LAYOUT_PANE_BASE& ParentPane,		//!< Parent pane
			ICONID iconid,									//!< Icon ID
			const MISTRING& tooltip,					//!< ToolTip string
			STYLE style = STYLE_Check,					//!< Toggle style
			bool dftvalue = false,						//!< Default value
			UINT16 iconsize = 0							//!< Icon size, 0 for default
			);

		//! Get current toggle setting.
		bool GetValue () const
			{ return (m_value); }

		//! Set delegate to call when button pressed.
		void SetDelegateOnPressed (
			DELEGATE_VOID_NOPARMS delegate
			) { m_DelegateOnPressed = delegate; }

		//! Set control value.
		void SetValue (
			bool value,										//!< Value to set
			bool notify = true							//!< If true call OnChangeValue() if value changed
			);

	private:
		#ifndef GENERATING_DOXYGEN_OUTPUT

		DELEGATE_VOID_NOPARMS m_DelegateOnPressed;

		bool m_value;

		static void CB_ToggleButton (Widget, CTRL_TOGGLEBUTTON*, void*);
		static void XT_EventHandler (Widget, CTRL_TOGGLEBUTTON*, XEvent*, void*);

		Pixmap m_PixmapOffNormal;
		Pixmap m_PixmapOffDisabled;
		Pixmap m_PixmapOffMouseOver;
		Pixmap m_PixmapOnNormal;
		Pixmap m_PixmapOnDisabled;
		Pixmap m_PixmapOnMouseOver;

		void CreateIconX (
			MGUI::LAYOUT_PANE_BASE& ParentPane,
			const MGUI::ICON& IconOff,
			const MGUI::ICON& IconOn,
			const MISTRING& tooltip,
			STYLE style,
			bool dftvalue
			);

		void CreateWidget (
			MGUI::LAYOUT_PANE_BASE& ParentPane,
			XTARGS& args,
			const char* label,
			STYLE style,
			bool dftvalue
			);

		CTRL_TOGGLEBUTTON (const CTRL_TOGGLEBUTTON&);
		CTRL_TOGGLEBUTTON& operator= (const CTRL_TOGGLEBUTTON&);
		#endif // GENERATING_DOXYGEN_OUTPUT

		// Overridables.

		//! Called when button is pushed.
		//! Default implementation calls delegate if has been set.
		virtual void OnPressed ();

	};
#ifndef GENERATING_DOXYGEN_OUTPUT
DEFINE_ENUM_OP_BITWISE(CTRL_TOGGLEBUTTON::STYLE)
#endif


//===================================================================================================================
//! Control to display non-editable multi-line text in optionally scrollable view.
class CTRL_TEXT : public MGUI::CTRL {
	public:

		//! Style for text control.
		enum STYLE {
			STYLE_NoScrollBars =		0x0000,
			STYLE_ScrollH =			0x0001,
			STYLE_ScrollV =			0x0002,
			STYLE_ScrollBoth =		(STYLE_ScrollH|STYLE_ScrollV),
			STYLE_WidthInPixels =	0x0004,
			STYLE_NoFrame =			0x0008,		//!< Don't include frame around control
			STYLE_NoMargin =			0x0010,		//!< Don't include margin around control
			};

		//! Constructor.
		CTRL_TEXT ()
			{ }

		//! Clear the text.
		void Clear (
			);

		//! Clear the current selection
		void ClearSelection ();

		//! Create control.
		void Create (
			MGUI::LAYOUT_PANE_BASE& ParentPane,		//!< Parent pane
			int width = 40,								//!< Width in characters or pixels per style flags
			int height = 10,								//!< Height in text lines
			STYLE style = STYLE_NoScrollBars,		//!< Scroll bar display style
			MGUI::LAYOUT_SIZEALIGN sizealign = MGUI::LAYOUT_SIZEALIGN_Expand
			);

		//! Select range of characters in string.
		void SetSelection (
			int StartChar = 0,						//!< Starting character position in string, 0 for beginning
			int EndChar = -1,							//!< Ending character position, -1 for end of string
			bool NoScroll = true						//!< Indicates whether to scroll caret into view
			);

		//! Set text to display.
		void SetString (
			const MISTRING& string						//!< String to display
			);

	private:
		#ifndef GENERATING_DOXYGEN_OUTPUT
		CTRL_TEXT (const CTRL_TEXT&);
		CTRL_TEXT& operator= (const CTRL_TEXT&);
		#endif // GENERATING_DOXYGEN_OUTPUT
	};
#ifndef GENERATING_DOXYGEN_OUTPUT
DEFINE_ENUM_OP_BITWISE(CTRL_TEXT::STYLE)
#endif

//===================================================================================================================
//! Separator control, draws horizontal or vertical line.
class CTRL_SEPARATOR : public MGUI::CTRL {
	public:

		enum CTRLSTYLE {
			CTRLSTYLE_Solid,								//!< Solid line
			CTRLSTYLE_Dashed,								//!< Dashed line
			CTRLSTYLE_ShadowEtchedIn,					//!< Double line giving the effect of a line etched into the window
			CTRLSTYLE_ShadowEtchedOut					//!< Double line giving the effect of an etched line coming out from the window
			};

		enum ORIENTATION {
			ORIENTATION_Automatic,						//!< Automatic orientation as opposite of parent pane fill order
			ORIENTATION_Horizontal,
			ORIENTATION_Vertical
			};

		//! Constructor.
		CTRL_SEPARATOR ();

		//! Destructor.
		~CTRL_SEPARATOR ();

		//! Create separator control.
		void Create (
			LAYOUT_PANE_BASE& ParentPane,				//!< Parent pane
			CTRLSTYLE type = CTRLSTYLE_ShadowEtchedIn,
			UINT8 thickness = 1,							//!< Total thickness in pixels, minimum of 2 for shadowed separators
			int length = 0,								//!< Length in pixels, 0 to extend to edges of parent pane
			ORIENTATION orientation = ORIENTATION_Automatic
			);

		//! Create separator control with text.
		void Create (
			LAYOUT_PANE_BASE& ParentPane,				//!< Parent pane
			const MISTRING& label,						//!< Label text string
			CTRLSTYLE type = CTRLSTYLE_ShadowEtchedIn,
			UINT8 thickness = 1							//!< Total thickness in pixels, minimum of 2 for shadowed separators
			);

	private:
		#ifndef GENERATING_DOXYGEN_OUTPUT
		CTRL_SEPARATOR (const CTRL_SEPARATOR&);
		CTRL_SEPARATOR& operator= (const CTRL_SEPARATOR&);
		#endif // GENERATING_DOXYGEN_OUTPUT
	};

//=====================================================================================================================

}	// End of MGUI namespace

#endif	// INC_MGUI_CTRL_H
