/**
 *	\file mi32/fileraw.h
 *	\brief Class FILE_RAW - Handle open files on supported platforms
 *
 * \if NODOC
 * $Id: fileraw.h_v 1.22 2012/08/29 15:50:05 mju Exp $
 *
 * $Log: fileraw.h_v $
 * Revision 1.22  2012/08/29 15:50:05  mju
 * Remove delegate hdr.
 *
 * Revision 1.21  2012/08/29 13:58:46  mju
 * Clean up dox.
 *
 * Revision 1.20  2006/07/07 22:49:25  scowan
 * Make dtor virtual.
 *
 * Revision 1.19  2006/07/06 19:37:30  scowan
 * Added comments about network files to be open and sharing values.
 *
 * Revision 1.11  2003/07/11 15:26:45  mju
 * Inherit from GENERICRW.
 * Eliminate error handler parms.
 *
 * Revision 1.1  2000/08/03 13:14:43  mju
 * Initial revision
 * \endif
**/

#ifndef	INC_MI32_FILERAW_H
#define	INC_MI32_FILERAW_H

#ifndef	INC_MI32_GENERICRW_H
	#include <mi32/genericrw.h>
#endif

#ifndef	INC_MI32_FILEACCESS_H
	#include <mi32/fileaccess.h>
#endif


#ifdef MISYSTEMDLL
	#define CLASSLIBEXPORT MI_DLLCLASSEXPORT
#else
	#define CLASSLIBEXPORT MI_DLLCLASSIMPORT
#endif


#ifndef GENERATING_DOXYGEN_OUTPUT
// Forward declarations.
class FILE_RAW_PRIV;
class FILEPATH;
#endif


//! Class for platform-independent "raw" file read/write.
//! This class is not thread safe.  Most of the time the instance is specific to a single thread.
//! If the instance is shared between threads, it must be guarded with a suitable locking method.
class CLASSLIBEXPORT FILE_RAW : public GENERICRW {
	public:

		//! Default constructor.
		FILE_RAW ();

		//! Destructor, closes file if open.
		virtual ~FILE_RAW ();

		//! Close file if open.
		void Close ();

		//! Get current path if file is open.
		//! @return 'true' if open, 'false' if not.
		bool GetPath (
			FILEPATH& path								//!< Path returned
			) const;

		//! Determine if file is currently open.
		bool IsOpen () const;

		//! Open or create specified file.
		//! If file is already open the previous file will be closed.
		//! If file is on a network file system, 'share' is treated as 'FILE_SHARE_Any', regardless of parameter setting.
		//! In that case, if a share mechanism is needed, you will need to implement your own scheme.
		ERRVALUE Open (
			const FILEPATH& filepath,					//!< Path to file to be opened or created
			FILE_ACCESS access = FILE_ACCESS_Read,	//!< Desired access
			FILE_SHARE share = FILE_SHARE_Any		//!< Sharing mode
			);

	private:
		#ifndef GENERATING_DOXYGEN_OUTPUT

		// Private/internal data done via pointer so if change implementation won't break DLL.
		FILE_RAW_PRIV *m_priv;

		// Overrides from GENERICRW
		virtual CAPABILITIES v_GetCapabilities () const;
		virtual INT64 v_GetSize () const;
		virtual INT64 v_GetSizeLimit () const;
		virtual INT32 v_Read (void* buffer, INT32 numbytes, bool AllowPartial);
		virtual ERRVALUE v_Resize (INT64 newsize);
		virtual INT64 v_Seek (INT64 offset, SEEKFROM from);
		virtual INT64 v_Tell () const;
		virtual ERRVALUE v_Write (const void* buffer, INT32 numbytes);

		FILE_RAW (const FILE_RAW&);
		FILE_RAW& operator= (const FILE_RAW&);

		#endif // GENERATING_DOXYGEN_OUTPUT
	};

#undef	CLASSLIBEXPORT
#endif
