/**
 * \file identifier.h <mi32/identifier.h>
 * \brief IDENTIFIER class definition
 *
 * \if NODOC
 * $Id: identifier.h_v 1.13 2005/01/13 17:17:56 mju Exp $
 *
 * $Log: identifier.h_v $
 * Revision 1.13  2005/01/13 17:17:56  mju
 * Add idlist.containsNoCase.
 *
 * Revision 1.12  2004/07/20 16:38:41  mju
 * Add set methods.
 *
 * Revision 1.11  2004/04/20 13:48:30  mju
 * Change IDENTIFIERLIST to class and add FindId method.
 *
 * Revision 1.10  2004/04/01 21:28:02  mju
 * Include mistring hdr.
 *
 * Revision 1.9  2004/04/01 20:43:15  mju
 * Add Assign methods using Latin1 and UTF8 strings.
 *
 * Revision 1.8  2003/09/29 16:25:56  mju
 * Add ctor/op= from string.
 *
 * Revision 1.7  2003/09/18 16:04:48  mju
 * Add getfullstr.
 *
 * Revision 1.6  2003/09/15 13:49:56  fileserver!dwilliss
 * Doxygen
 *
 * Revision 1.5  2003/09/03 20:13:04  mju
 * Add operator<.
 *
 * Revision 1.4  2003/08/20 19:25:10  mju
 * Add default ctor and CreateFromXML method.
 *
 * Revision 1.3  2003/08/18 20:01:50  mju
 * Fix template instantiation.
 *
 * Revision 1.2  2003/08/18 17:43:50  mju
 * Add op==/op!=.
 *
 * Revision 1.1  2003/08/18 14:37:05  mju
 * Initial revision
 * \endif
**/

#ifndef	INC_MI32_IDENTIFIER_H
#define	INC_MI32_IDENTIFIER_H

#ifndef	INC_MI32_CODESPACE_H
#include <mi32/codespace.h>
#endif

#ifndef	INC_MI32_MISTRING_H
#include <mi32/mistring.h>
#endif

#ifndef	INC_MI32_STRUTF8_H
#include <mi32/strutf8.h>
#endif

#ifndef	INC_MI32_MILIST_H
#include <mi32/milist.h>
#endif

#ifndef GENERATING_DOXYGEN_OUTPUT
class XMLNODE;
#endif // GENERATING_DOXYGEN_OUTPUT

#ifdef MISYSTEMDLL
	#define CLASSLIBEXPORT MI_DLLCLASSEXPORT
#else
	#define CLASSLIBEXPORT MI_DLLCLASSIMPORT
#endif


//! Identifier container used to reference specific persistent object instance.
class CLASSLIBEXPORT IDENTIFIER {
	public:

		//! Default constructor.
		IDENTIFIER (
			);

		//! Construct using numeric code.
		IDENTIFIER (
			CODESPACE codespace,
			INT32 codenum
			);

		//! Construct using string code in specified codespace.
		//! If string contains a single integer value it will be converted to a number.
		IDENTIFIER (
			CODESPACE codespace,
			const char *codestr
			);

		//! Construct using string code.
		//! If string contains a single integer value it will be converted to a number.
		IDENTIFIER (
			const char *codestr
			);

		//! Destructor.
		~IDENTIFIER (
			);

		//! Assign from  string code.
		//! If string contains a single integer value it will be converted to a number.
		IDENTIFIER& operator= (
			const char *codestr					//!< Code string in form "codespacename:code", Latin1 encoding
			);

		#ifndef GENERATING_DOXYGEN_OUTPUT
		//! Used by MILIST::Sort().
		bool operator< (
			const IDENTIFIER& rhs
			) const;
		#endif

		//! Assign from string code.
		//! If string contains a single integer value it will be converted to a number.
		ERRVALUE Assign (
			const char *codestr					//!< Code string in form "codespacename:code", Latin1 encoding
			);

		//! Assign from string code.
		//! If string contains a single integer value it will be converted to a number.
		ERRVALUE Assign (
			const UINT8 *codestr					//!< Code string in form "codespacename:code", UTF8 encoding
			);

	#ifndef NODLLLINKAGE
		//! Create identifier from XML node.
		//! Node must be in the form:
		//! <tt>
		//!  \<nodename\>codespacename:code\</nodename\>
		//! </tt>
		//! The nodename itself is not relevent.
		//! If the codespace is not specified the CODESPACE_MicroImages will be used.
		ERRVALUE CreateFromXML (
			const XMLNODE *xmlnode
			);
	#endif

		//! Get codespace enumeration.
		CODESPACE GetCodespace (
			) const;

		//! Get full identifier string in form "codespace:id".
		MISTRING GetFullStr (
			) const;

		//! Get code number.
		//! @return Code number, 0 if not numeric.
		INT32 GetNum (
			) const;

		//! Get code string.
		//! @return Code string
		const STRUTF8& GetStr (
			) const;

		//! Set codespace and numeric code.
		void Set (
			CODESPACE codespace,
			INT32 num
			);

		//! Set codespace and string code.
		void Set (
			CODESPACE codespace,
			const char *code
			);

		#ifndef GENERATING_DOXYGEN_OUTPUT
		// Comparison for equality.
		bool IsEqual (const IDENTIFIER& rhs) const;
		bool IsEqualNoCase (const IDENTIFIER& rhs) const;
		#endif

	private:
		#ifndef GENERATING_DOXYGEN_OUTPUT
		CODESPACE m_codespace;
		INT32 m_num;
		STRUTF8 m_str;

		void SetNumFromStr ();
		#endif // GENERATING_DOXYGEN_OUTPUT
	};

inline bool operator== (
	const IDENTIFIER& lhs,
	const IDENTIFIER& rhs
	) { return (lhs.IsEqual(rhs)); }

inline bool operator!= (
	const IDENTIFIER& lhs,
	const IDENTIFIER& rhs
	) { return (!lhs.IsEqual(rhs)); }

/*
#if defined(WIN32) && !defined(MISYSTEMDLL) && !defined(NODLLLINKAGE)
#pragma warning(disable:4231)
extern template class CLASSLIBEXPORT MILIST<IDENTIFIER>;		// Template instantiation
#pragma warning(default:4231)
#endif
typedef MILIST<IDENTIFIER> IDENTIFIERLIST;
*/

class CLASSLIBEXPORT IDENTIFIERLIST : public MILIST<IDENTIFIER> {
	public:
		//! Determine if list contains identifier with case-insensitive name comparison.
		//! @return true if found, false if not.
		bool ContainsNoCase (
			const IDENTIFIER& id				//! Identifier to look for
			) const;
			
		//! Find identifier in list with specified codespace.
		//! @return true if match found, false if not
		bool FindId (
			CODESPACE codespace,
			IDENTIFIER& id
			) const;
	};

#undef	CLASSLIBEXPORT

#endif	// INC_MI32_IDENTIFIER_H
