/*****************************************************************************
 *
 * \file mempool.h <mi32/mempool.h>
 * \brief A memory pool management class
 *
 * \if NODOC
 * $Log: mempool.h_v $
 * Revision 1.10  2006/10/12 17:21:03  mju
 * Remove unneeded hdrs.
 *
 * Revision 1.9  2005/11/11 21:36:59  dwilliss
 * Gave MEMPOOLLOW a copy ctor and assignment operator.  They just increment
 * refcount on the private data
 *
 * Revision 1.8  2005/11/11 16:08:36  dwilliss
 * added ctor with size hint
 *
 * Revision 1.7  2003/11/14 18:46:56  dwilliss
 * Don't use _T in templates. Some Unix systems define it in system includes
 *
 * Revision 1.6  2003/09/15 13:49:56  fileserver!dwilliss
 * Doxygen
 *
 * Revision 1.5  2003/07/09 14:36:37  mju
 * Use standard name for classlibexport.
 *
 * Revision 1.4  2002/12/03 17:08:11  dwilliss
 * Changed API so that Alloc,Realloc and Free take references to pointers.
 * Used template methods to avoid need to cast.
 * Implementation class exported from DLL modified so that all data is in
 *   a PRIVDATA so that the sizeof(MEMPOOLLOW) will never have to change
 * Added a pointer to STATS structure so that if compiled with DEBUG #defined,
 *   (that is, misystem is compiled with DEBUG #defined), you can monitor
 *   the pool's memory usage in the debugger by expanding m_stats.
 *
 * Revision 1.3  2002/12/02 17:24:40  dwilliss
 * Finalized API.  Added Realloc method.  Gave both alloc methods a bClear
 * parameter (optional)
 *
 * Revision 1.2  2002/12/02 16:19:41  dwilliss
 * Document the class itself.
 *
 * Revision 1.1  2002/12/02 16:05:00  dwilliss
 * Initial revision
 *
 *
 * \endif
 ****************************************************************************/

#ifndef INC_MI32_MEMPOOL_H
#define INC_MI32_MEMPOOL_H

#ifndef INC_MI32_STDDEFNS_H
#include <mi32/stddefns.h>
#endif

#ifdef MISYSTEMDLL
 #define CLASSLIBEXPORT MI_DLLCLASSEXPORT
#else
 #define CLASSLIBEXPORT MI_DLLCLASSIMPORT
#endif

#ifndef GENERATING_DOXYGEN_OUTPUT
//! A class for quickly allocating many small data structures of the same size.
//!
//! This class is optimized for allocating a lot of small (< 1K each) data 
//! structures.  It allocates larger block keeps track of free-lists itself.
class CLASSLIBEXPORT MEMPOOLLOW {
	protected:

		//! Constructor
		MEMPOOLLOW();

		//! Constructor.
		//! This version allows us to tell the pool the most common size we
		//! will use.
		MEMPOOLLOW (
			INT32 SizeHint
			);

		//! Copy constructor
		MEMPOOLLOW (
			const MEMPOOLLOW& rhs
			);

		//! Destructor.
		//!
		//! Note:  Destroying a MEMPOOL automatically frees anything allocated
		//! through it.
		~MEMPOOLLOW();


		MEMPOOLLOW& operator= (const MEMPOOLLOW& rhs);

		//! Allocate memory from the mempool.  
		//!
		//! IMPORTANT!  Any memory allocated this way must be freed by
		//! calling MEMPOOL::Free(), not MmFree(), free() or delete.
		ERRVALUE AllocLow (
			void** ptr, 
			UINT32 size,
			bool bClear = false		//!< Clear to 0 if true
			);

		//! Resize a memory block in the mempool.
		//!
		//! IMPORTANT!  Any memory allocated this way must be freed by
		//! calling MEMPOOL::Free(), not MmFree(), free() or delete.
		//! Also, you can only a pointer which was allocated from the same pool.
		ERRVALUE ReallocLow (
			void** ptr, 
			UINT32 size,
			bool bClear = false //!< Clear the new part to 0 if true
			);

		void SetSizeHintLow (
			UINT32 size
			);

		//! Free a chunk of memory allocated from a MEMPOOL
		//!
		//! Since the allocated chunk knows which pool it came from, 
		//! this method can be static.  This means you can free the memory
		//! from a destructor which doesn't know anything about the MEMPOOL.
		static void FreeLow (
			void* ptr
			);


	private:
	   #ifndef GENERATING_DOXYGEN_OUTPUT
		class PRIVDATA;
		class BUCKET;
		struct STATS {
			UINT32 TotalAlloc;
			UINT32 FreeSpace;
			};
		
		PRIVDATA* m_priv;		//!< So we can expand at will without changing size
		STATS* m_stats;		//!< Will be NULL unless misystem is compiled for DEBUG. Then m_stats can be expanded in the debugger.
		#endif // GENERATING_DOXYGEN_OUTPUT

	};

#endif //!< GENERATING_DOXYGEN_OUTPUT

//! A class for quickly allocating many small data structures of the same size.
//!
//! This class is optimized for allocating a lot of small (< 1K each) data 
//! structures.  It allocates larger block keeps track of free-lists itself.
//!
//! The class that MEMPOOL is derived from is exported from misystem.dll
//! MEMPOOL itself is not exported and consists mostly of template methods
//! (which can't be exported from a DLL)
class MEMPOOL : public MEMPOOLLOW {
	public:
		//! Constructor
		MEMPOOL() {}

		//! Constructor
		//! This version allows us to tell the pool the most common size we
		//! will use.
		MEMPOOL (
			INT32 SizeHint
			) :
			MEMPOOLLOW(SizeHint)
			{}

		//! Destructor.
		//!
		//! Note:  Destroying a MEMPOOL automatically frees anything allocated
		//! through it.
		~MEMPOOL() {}

		//! Allocate memory from the mempool.  
		//!
		//! IMPORTANT!  Any memory allocated this way must be freed by
		//! calling MEMPOOL::Free(), not MmFree(), free() or delete.
		//!
		//! Also note:  When calling MmAlloc(), you had to do this...
		//! 	MmAlloc((void**)&ptr, size);
		//! Now you must leave off the (void**)&, leaving...
		//! 	MemPool.Alloc(ptr, size);
		//! Be very careful to remove the & too, or you'll end up with someting
		//! that compiles but doesn't do what you want.
		template <class _CT> ERRVALUE Alloc (
			_CT*& ptr, 
			UINT32 size,
			bool bClear = false		//!< Clear to 0 if true
			) {
			return (AllocLow((void**)&ptr, size, bClear));
			}

		//! Resize a memory block in the mempool.
		//!
		//! IMPORTANT!  Any memory allocated this way must be freed by
		//! calling MEMPOOL::Free(), not MmFree(), free() or delete.
		//! Also, you can only reallocate a pointer which was allocated 
		//! from the same pool.
		//!
		//! Since the memory to be reallocated knows how big it is, you
		//! don't have to pass the old size, even if clearing the new parts.
		//!
		//! Also note:  When calling MmAlloc(), you had to do this...
		//! 	MmAlloc((void**)&ptr, size);
		//! Now you must leave off the (void**)&, leaving...
		//! 	MemPool.Alloc(ptr, size);
		//! Be very careful to remove the & too, or you'll end up with someting
		//! that compiles but doesn't do what you want.
		template <class _CT> ERRVALUE Realloc (
			_CT*& ptr, 
			UINT32 size,
			bool bClear = false //!< Clear the new part to 0 if true
			) {
			return (ReallocLow((void**)&ptr, size, bClear));
			}

		//! Set a default allocation size.
		//!
		//! By default, a MEMPOOL will round any allocation size up to the nearest
		//! power of 2.  You can be more space effeciant by giving it a size hint.
		//! This will create a bucket for the specified size so that it will
		//! waste less RAM.  NOTE! You can only have one size hint!  If you try
		//! to set multiple hints, it will free anything allocated at the old
		//! size.
		void SetSizeHint (
			UINT32 size
			) {
			SetSizeHintLow(size);
			}


		//! Free a chunk of memory allocated from a MEMPOOL
		//!
		//! Since the allocated chunk knows which pool it came from, 
		//! this method can be static.  This means you can free the memory
		//! from a destructor which doesn't know anything about the MEMPOOL.
		//!
		//! Note, the pointer will be set to NULL after freeing, and freeing
		//! a NULL pointer is harmless.
		template<class _CT> static void Free (
			_CT*& ptr
			) {
			FreeLow(ptr);
			ptr = 0;
			}


	};


#undef CLASSLIBEXPORT

#endif


