/******************************************************************************
 *
 * \file mi32/mijob.h
 * \brief MIJOB
 *
 * \if NODOC
 * $Log: mijob.h_v $
 * Revision 1.33  2012/10/11 20:47:22  bcolombe
 * Add blocking by job id
 *
 * Revision 1.32  2012/06/18 17:59:49  mju
 * Add add/findArray for bool/UINT8/UINT16.
 *
 * Revision 1.31  2012/06/11 19:57:58  bcolombe
 * Support serializable object
 *
 * Revision 1.30  2011/05/20 14:22:39  mju
 * Use proper inclusion guard naming convention.
 *
 * Revision 1.29  2011/05/19 21:14:13  bcolombe
 * Add ResolveTempJobs
 *
 * Revision 1.28  2011/04/29 16:43:34  bcolombe
 * Make CreateGroup static
 *
 * Revision 1.27  2011/04/06 22:06:06  bcolombe
 * Add GetPath
 *
 * Revision 1.26  2011/03/31 22:20:50  bcolombe
 * Create and Write group in single method
 *
 * Revision 1.25  2011/03/28 19:24:53  bcolombe
 * Add group support
 *
 * Revision 1.24  2010/10/07 17:45:36  bcolombe
 * Add option to block if file exists or doesnt exist
 *
 * Revision 1.23  2010/04/23 20:51:08  mju
 * Add add/findBinary.
 *
 * Revision 1.22  2010/01/08 18:01:09  bcolombe
 * Add option for adding input/output that won't be checked for blocking issues
 *
 * Revision 1.21  2010/01/06 17:11:20  mju
 * Fix findEnum shadowing error.
 *
 * Revision 1.20  2010/01/06 15:47:47  mju
 * Add findEnum that returns xmlnode, and node parm to getEnum.
 *
 * Revision 1.19  2010/01/04 17:25:12  bcolombe
 * Default priority is now 5
 *
 * Revision 1.18  2009/06/22 16:29:37  bcolombe
 * Add char* value
 *
 * Revision 1.17  2009/06/22 14:16:52  bcolombe
 * Remove dlg_schedule from mijob
 *
 * Revision 1.16  2009/06/19 20:19:31  mju
 * Change statusmgr to pointer so not constructed until needed.
 * Don't include hdrs not explicitly needed, forward-declare classes instead.
 *
 * Revision 1.15  2009/06/19 14:01:12  mju
 * Add int/long variants of findValue.
 * Add template methods for getEnum/Defaulted and getValueDefaulted.
 *
 * Revision 1.14  2009/06/18 17:49:55  mju
 * Add setReady.
 *
 * Revision 1.13  2009/06/18 17:40:16  bcolombe
 * Add ability to write scheduled job directly
 *
 * Revision 1.12  2009/06/18 16:13:47  mju
 * Add addInput/OutputPath variants using filepath.
 *
 * Revision 1.11  2009/06/17 17:42:52  bcolombe
 * Added changed for scheduled jobs
 *
 * Revision 1.10  2009/05/01 14:38:32  mju
 * Add findPath variant using FILEPATH.
 *
 * Revision 1.9  2009/04/30 22:49:37  bcolombe
 * Make destruc. virtual and exclude private members from doxygen
 *
 * Revision 1.8  2009/02/12 22:51:32  bcolombe
 * Enable job status on mac
 *
 * Revision 1.7  2009/02/04 23:11:12  bcolombe
 * Disabled jobstatus on mac
 *
 * Revision 1.6  2009/01/23 19:36:18  bcolombe
 * Added way to read without starting up status reporting
 *
 * Revision 1.5  2009/01/23 18:57:51  bcolombe
 * Added job status
 *
 * Revision 1.4  2008/11/21 23:12:58  bcolombe
 * Add to write the ability to create as held
 *
 * Revision 1.3  2008/11/06 18:13:03  bcolombe
 * Added GetInput/OuputList, AddInput/OutputPath, FindPath
 *
 * Revision 1.2  2008/10/02 21:06:00  bcolombe
 * Added GetRunParmsNode
 *
 * Revision 1.1  2008/10/02 18:02:51  bcolombe
 * Initial revision
 *
 * \endif
*/
/*****************************************************************************/

#ifndef INC_MI32_MIJOB_H
#define INC_MI32_MIJOB_H

#ifndef INC_MI32_XML_H
#include <mi32/xml.h>
#endif

#ifndef INC_MI32_FILEPATH_H
#include <mi32/filepath.h>
#endif

#ifndef GENERATING_DOXYGEN_OUTPUT
// Forward declarations.
class FILEPATHLIST;
class MIJOBSTATUS;
class SERIALIZABLE;
namespace RVC {
	class OBJITEM;
	}
#endif


class MIJOB {
	public:

		//! Write group file to default job directory
		static ERRVALUE CreateGroup (
			const MISTRING& description,			//!< Job description string
			MISTRING& GroupID,						//!< GroupID created
			INT32 priority = -1,						//!< Group priority
			bool bCreateAsTemp = false				//!< Create as temp group
			);

		//! Write group file to default job directory
		static ERRVALUE CreateGroup (
			const FILEPATH& directory,				//!< Directory to create job file in
			const MISTRING& description,			//!< Job description string
			MISTRING& GroupID,						//!< GroupID created
			INT32 priority = -1,						//!< Group priority
			bool bCreateAsTemp = false				//!< Create as temp group
			);

		//! Make temporary jobs actual jobs
		static ERRVALUE ResolveTempJobs ();

		//! Constructor.
		MIJOB ();

		//! Destructor.
		virtual ~MIJOB ();
		
		//! Add 'bool' array data value.
		XMLNODE* AddArray (
			const char *name,							//!< Name of variable
			const SIMPLE_ARRAY<bool>& data,		//!< Array data
			XMLNODE *node = 0							//!< Parent node
			);

		//! Add 'UINT8' array data value.
		XMLNODE* AddArray (
			const char *name,							//!< Name of variable
			const SIMPLE_ARRAY<UINT8>& data,		//!< Array data
			XMLNODE *node = 0							//!< Parent node
			);

		//! Add 'UINT16' array data value.
		XMLNODE* AddArray (
			const char *name,							//!< Name of variable
			const SIMPLE_ARRAY<UINT16>& data,	//!< Array data
			XMLNODE *node = 0							//!< Parent node
			);

		//! Add binary data value.
		XMLNODE* AddBinary (
			const char *name,				//!< Name of variable
			const SIMPLE_ARRAY<UINT8>& data,
			XMLNODE *node = 0				//!< Parent node
			);

		//! Block this job from running until provided job completes.
		void AddBlockID (
			const MISTRING& JobID
			);

		//! Read list of blocking job IDs
		void GetBlockIDs (
			MISTRINGLIST& BlockIDs
			);

		//! Create input object.  Job Queue will treat it as having read lock.
		XMLNODE* AddInputObject (
			const char* id,							//!< ID string assigned to object, should choose something specific to process
			const RVC::OBJITEM& objitem,			//!< Objitem of object
			XMLNODE *node = 0,						//!< Parent node
			bool IgnoreForBlocking = false,		//!< If set to true, will only be used for tooltips and not checked for blocking issues
			bool BlockIfExist = false,				//!< If set to true, will block job if file exists
			bool BlockIfNotExist = false			//!< If set to true, will block job if file does not exist
			);

		//! Create input path using MISTRING.  Job Queue will treat it as having read lock.
		XMLNODE* AddInputPath (
			const char* id,							//!< ID string assigned to object, should choose something specific to process
			const MISTRING& path,					//!< String of path
			XMLNODE *node = 0,						//!< Parent node
			bool IgnoreForBlocking = false,		//!< If set to true, will only be used for tooltips and not checked for blocking issues
			bool BlockIfExist = false,				//!< If set to true, will block job if file exists
			bool BlockIfNotExist = false			//!< If set to true, will block job if file does not exist
			);

		//! Create input path using char*.  Job Queue will treat it as having read lock.
		XMLNODE* AddInputPath (
			const char* id,							//!< ID string assigned to object, should choose something specific to process
			const char* path,							//!< String of path
			XMLNODE *node = 0,						//!< Parent node
			bool IgnoreForBlocking = false,		//!< If set to true, will only be used for tooltips and not checked for blocking issues
			bool BlockIfExist = false,				//!< If set to true, will block job if file exists
			bool BlockIfNotExist = false			//!< If set to true, will block job if file does not exist
			);

		//! Create input path using FILEPATH.  Job Queue will treat it as having read lock.
		XMLNODE* AddInputPath (
			const char* id,							//!< ID string assigned to object, should choose something specific to process
			const FILEPATH& path,					//!< Path
			XMLNODE *node = 0,						//!< Parent node
			bool IgnoreForBlocking = false,		//!< If set to true, will only be used for tooltips and not checked for blocking issues
			bool BlockIfExist = false,				//!< If set to true, will block job if file exists
			bool BlockIfNotExist = false			//!< If set to true, will block job if file does not exist
			);

		//! Add general node
		//! @return Node to contain
		XMLNODE* AddNode (
			const char *name,							//!< Name of node
			XMLNODE *node = 0							//!< Parent node
			);

		//! Create output object.  Job Queue will treat it as having write lock.
		XMLNODE* AddOutputObject (
			const char* id,							//!< ID string assigned to object, should choose something specific to process
			const RVC::OBJITEM& objitem,			//!< Objitem of object
			XMLNODE *node = 0,						//!< Parent node
			bool IgnoreForBlocking = false		//!< If set to true, will only be used for tooltips and not checked for blocking issues
			);

		//! Create output path using MISTRING.  Job Queue will treat it as having write lock.
		XMLNODE* AddOutputPath (
			const char* id,					//!< ID string assigned to object, should choose something specific to process
			const MISTRING& path,			//!< String of path
			XMLNODE *node = 0,				//!< Parent node
			bool IgnoreForBlocking = false	//!< If set to true, will only be used for tooltips and not checked for blocking issues
			);

		//! Create output path using MISTRING.  Job Queue will treat it as having write lock.
		XMLNODE* AddOutputPath (
			const char* id,					//!< ID string assigned to object, should choose something specific to process
			const char* path,				//!< String of path
			XMLNODE *node = 0,				//!< Parent node
			bool IgnoreForBlocking = false	//!< If set to true, will only be used for tooltips and not checked for blocking issues
			);

		//! Create output path using FILEPATH.  Job Queue will treat it as having write lock.
		XMLNODE* AddOutputPath (
			const char* id,					//!< ID string assigned to object, should choose something specific to process
			const FILEPATH& path,			//!< Path
			XMLNODE *node = 0,					//!< Parent node
			bool IgnoreForBlocking = false	//!< If set to true, will only be used for tooltips and not checked for blocking issues
			);

		//! Add serializable object
		XMLNODE* AddSerializable (
			const char* name,					//!< Name of node
			const SERIALIZABLE& serializable,	//!< Serializable to add
			XMLNODE *node = 0					//!< Parent node
			);

		//! Add string variable
		XMLNODE* AddValue (
			const char *name,				//!< Name of variable
			const MISTRING& value,			//!< Value to assign
			XMLNODE *node = 0			//!< Parent node
			);

		//! Add char*
		XMLNODE* AddValue (
			const char *name,				//!< Name of variable
			const char* value,				//!< Value to assign
			XMLNODE *node = 0				//!< Parent node
			);

		//! Add UINT32 variable
		XMLNODE* AddValue (
			const char *name,				//!< Name of variable
			UINT32 value,					//!< Value to assign
			XMLNODE *node = 0			//!< Parent node
			);

		//! Add INT32 variable
		XMLNODE* AddValue (
			const char *name,				//!< Name of variable
			INT32 value,					//!< Value to assign
			XMLNODE *node = 0			//!< Parent node
			);

		//! Add UINT16 variable
		XMLNODE* AddValue (
			const char *name,				//!< Name of variable
			UINT16 value,					//!< Value to assign
			XMLNODE *node = 0			//!< Parent node
			);

		//! Add INT16 variable
		XMLNODE* AddValue (
			const char *name,				//!< Name of variable
			INT16 value,					//!< Value to assign
			XMLNODE *node = 0			//!< Parent node
			);

		//! Add UINT8 variable
		XMLNODE* AddValue (
			const char *name,				//!< Name of variable
			UINT8 value,					//!< Value to assign
			XMLNODE *node = 0			//!< Parent node
			);

		//! Add INT8 variable
		XMLNODE* AddValue (
			const char *name,				//!< Name of variable
			INT8 value,						//!< Value to assign
			XMLNODE *node = 0			//!< Parent node
			);

		//! Add double variable
		XMLNODE* AddValue (
			const char *name,				//!< Name of variable
			double value,					//!< Value to assign
			XMLNODE *node = 0			//!< Parent node
			);

		//! Add bool variable
		XMLNODE* AddValue (
			const char *name,				//!< Name of variable
			bool value,						//!< Value to assign
			XMLNODE *node = 0			//!< Parent node
			);

		//! Create new job file
		//! Filename will be YYYMMDD_HHMMSS_##.tntjob
		//! where ## gets incremented when there is already a job with same timestamp
		ERRVALUE CreateJob (
			const MISTRING& description,	//!< Job description string
			const MISTRING& process,		//!< String with executable and process to run, i.e. rastanly compsasjob
			INT32 priority = -1				//!< Job priority; 1-9
			);

		//! Get 'bool' array data variable.
		//! @return XML node containing value if found, NULL if not.
		XMLNODE* FindArray (
			const char *name,							//!< Name of variable	
			SIMPLE_ARRAY<bool>& data,				//!< Data returned
			const XMLNODE *node = 0					//!< Parent node or previous list node
			) const;

		//! Get 'UINT8' array data variable.
		//! @return XML node containing value if found, NULL if not.
		XMLNODE* FindArray (
			const char *name,							//!< Name of variable	
			SIMPLE_ARRAY<UINT8>& data,				//!< Data returned
			const XMLNODE *node = 0					//!< Parent node or previous list node
			) const;

		//! Get 'UINT16' array data variable.
		//! @return XML node containing value if found, NULL if not.
		XMLNODE* FindArray (
			const char *name,							//!< Name of variable	
			SIMPLE_ARRAY<UINT16>& data,			//!< Data returned
			const XMLNODE *node = 0					//!< Parent node or previous list node
			) const;

		//! Get binary data variable.
		//! @return XML node containing value if found, NULL if not.
		XMLNODE* FindBinary (
			const char *name,					//!< Name of variable	
			SIMPLE_ARRAY<UINT8>& data,		//!< Data returned
			const XMLNODE *node = 0			//!< Parent node or previous list node
			) const;

		//! Find enum value.
		//! Value is left unchanged if not found.
		//! @return XML node containing value if found, NULL if not.
		template <typename _ENUMTYPE> XMLNODE* FindEnum (
			const char *name,				//!< Variable name
			_ENUMTYPE& value,				//!< Value returned
			const XMLNODE *node = 0		//!< Parent node or previous list node
			) const { INT32 temp; XMLNODE *noderet = FindValue(name,temp,node); if (noderet != 0) value = static_cast<_ENUMTYPE>(temp); return (noderet); }

		//! Find general node.
		//! @return Node found
		XMLNODE* FindNode (
			const char *name,				//!< Name of node
			const XMLNODE *node = 0			//!< Parent node or previous list node
			) const;

		//! Get input or output object
		XMLNODE* FindObject (
			const char* id,					//!< ID of object
			RVC::OBJITEM& objitem,			//!< objitem returned
			const XMLNODE *node = 0			//!< Parent node or previous list node
			) const;

		//! Get input or output path as MISTRING.
		XMLNODE* FindPath (
			const char* id,					//!< ID of object
			MISTRING& pathstr,				//!< String path returned
			const XMLNODE *node = 0			//!< Parent node or previous list node
			) const;

		//! Get input or output path as FILEPATH.
		XMLNODE* FindPath (
			const char* id,					//!< ID of object
			FILEPATH& path,					//!< Path returned
			const XMLNODE *node = 0			//!< Parent node or previous list node
			) const;

		//! Find serializable object
		XMLNODE* FindSerializable (
			const char* id,					//!< ID of object
			SERIALIZABLE& serializable,		//!< Serializable returned
			const XMLNODE *node = 0			//!< Parent node or previous list node
			);

		//! Get string variable
		//! Value is left unchanged if content is empty or unrecognized.
		//! @return XML node containing value if found, NULL if not.
		XMLNODE* FindValue (
			const char *name,					//!< Name of variable	
			MISTRING& value,					//!< Value returned
			const XMLNODE *node = 0			//!< Parent node or previous list node
			) const;

		//! Get char*
		//! Value is left unchanged if content is empty or unrecognized.
		//! @return XML node containing value if found, NULL if not.
		XMLNODE* FindValue (
			const char *name,					//!< Name of variable	
			char*& value,					//!< Value returned
			const XMLNODE *node = 0			//!< Parent node or previous list node
			) const;

		//! Get 'unsigned long' variable
		//! Value is left unchanged if content is empty or unrecognized.
		//! @return XML node containing value if found, NULL if not.
		XMLNODE* FindValue (
			const char *name,					//!< Name of variable
			unsigned long& value,			//!< Value returned
			const XMLNODE *node = 0			//!< Parent node or previous list node
			) const;

		//! Get 'long' variable
		//! Value is left unchanged if content is empty or unrecognized.
		//! @return XML node containing value if found, NULL if not.
		XMLNODE* FindValue (
			const char *name,					//!< Name of variable
			long& value,						//!< Value returned
			const XMLNODE *node = 0			//!< Parent node or previous list node
			) const;

		//! Get 'unsigned int' variable
		//! Value is left unchanged if content is empty or unrecognized.
		//! @return XML node containing value if found, NULL if not.
		XMLNODE* FindValue (
			const char *name,					//!< Name of variable
			unsigned int& value,				//!< Value returned
			const XMLNODE *node = 0			//!< Parent node or previous list node
			) const;

		//! Get 'int' variable
		//! Value is left unchanged if content is empty or unrecognized.
		//! @return XML node containing value if found, NULL if not.
		XMLNODE* FindValue (
			const char *name,					//!< Name of variable
			int& value,							//!< Value returned
			const XMLNODE *node = 0			//!< Parent node or previous list node
			) const;

		//! Get UINT16 variable
		//! Value is left unchanged if content is empty or unrecognized.
		//! @return XML node containing value if found, NULL if not.
		XMLNODE* FindValue (
			const char *name,					//!< Name of variable
			UINT16& value,						//!< Value returned
			const XMLNODE *node = 0			//!< Parent node or previous list node
			) const;

		//! Get INT16 variable
		//! Value is left unchanged if content is empty or unrecognized.
		//! @return XML node containing value if found, NULL if not.
		XMLNODE* FindValue (
			const char *name,					//!< Name of variable
			INT16& value,						//!< Value returned
			const XMLNODE *node = 0			//!< Parent node or previous list node
			) const;

		//! Get UINT8 variable
		//! Value is left unchanged if content is empty or unrecognized.
		//! @return XML node containing value if found, NULL if not.
		XMLNODE* FindValue (
			const char *name,					//!< Name of variable
			UINT8& value,						//!< Value returned
			const XMLNODE *node = 0			//!< Parent node or previous list node
			) const;

		//! Get INT8 variable
		//! Value is left unchanged if content is empty or unrecognized.
		//! @return XML node containing value if found, NULL if not.
		XMLNODE* FindValue (
			const char *name,					//!< Name of variable
			INT8& value,						//!< Value returned
			const XMLNODE *node = 0			//!< Parent node or previous list node
			) const;

		//! Get double variable
		//! Value is left unchanged if content is empty or unrecognized.
		//! @return XML node containing value if found, NULL if not.
		XMLNODE* FindValue (
			const char *name,					//!< Name of variable
			double& value,						//!< Value returned
			const XMLNODE *node = 0			//!< Parent node or previous list node
			) const;

		//! Get float variable
		//! Value is left unchanged if content is empty or unrecognized.
		//! @return XML node containing value if found, NULL if not.
		XMLNODE* FindValue (
			const char *name,					//!< Name of variable
			float& value,						//!< Value returned
			const XMLNODE *node = 0			//!< Parent node or previous list node
			) const;

		//! Get bool variable
		//! Value is left unchanged if content is empty or unrecognized.
		//! @return XML node containing value if found, NULL if not.
		XMLNODE* FindValue (
			const char *name,				//!< Name of variable
			bool& value,					//!< Value returned
			const XMLNODE *node = 0		//!< Parent node or previous list node
			) const;

		//! Get job description
		void GetDescription (
			MISTRING& desc					//!< Description string returned
			) const;

		//! Get enum value.
		//! @return true if found, false if not.
		template <typename _ENUMTYPE> bool GetEnum (
			const char *name,				//!< Variable name
			_ENUMTYPE& value,				//!< Value returned
			const XMLNODE *node = 0		//!< Parent node or previous list node
			) const { INT32 temp; if (FindValue(name,temp,node) == 0) return (false); value = static_cast<_ENUMTYPE>(temp); return (true); }

		//! Get enum value with default.
		//! @return Value if found, default if not.
		template <typename _ENUMTYPE> _ENUMTYPE GetEnumDefaulted (
			const char *name,				//!< Variable name
			_ENUMTYPE dft,					//!< Default value
			const XMLNODE *node = 0		//!< Parent node or previous list node
			) const { INT32 temp; if (!FindValue(name,temp,node)) return (dft); else return (static_cast<_ENUMTYPE>(temp)); }

		//! Get list of filepaths used for input, only including those marked as needing to be checked for blocking issues
		void GetBlockingInputList (
			FILEPATHLIST& list
			) const;

		//! Get list of filepaths used for output, only including those marked as needing to be checked for blocking issues
		void GetBlockingOutputList (
			FILEPATHLIST& list
			) const;

		//! Get list of filepaths that will block if exist
		void GetBlockingIfExistList (
			FILEPATHLIST& list
			) const;

		//! Get list of filepaths that will block if dont exist
		void GetBlockingIfNotExistList (
			FILEPATHLIST& list
			) const;

		//! Get id of group job belongs to
		void GetGroupID (
			MISTRING& GroupID
			);

		//! Get list of filepaths used for input, for tooltips
		void GetInputList (
			FILEPATHLIST& list
			) const;

		//! Get list of filepaths used for output, for tooltips
		void GetOutputList (
			FILEPATHLIST& list
			) const;

		//! Get path job read from
		const FILEPATH& GetPath () const {return m_path;}

		//! Get job priority
		INT32 GetPriority () const;

		//! Get process string
		void GetProcess (
			MISTRING& process				//!< Process string returned
			) const;

		//! Get parms node
		XMLNODE* GetRunParmsNode () {return m_runparmsnode;}

		//! Get root node
		XMLNODE* GetRootNode () {return m_rootnode;}

		//! Get numeric value with default.
		//! @return Value if found or default specified.
		template <typename _NUMTYPE> _NUMTYPE GetValueDefaulted (
			const char *name,				//!< Variable name
			_NUMTYPE value					//!< Default value
			) const { FindValue(name,value); return (value); }

		//! Check if job is blocking with provided job
		bool IsBlockedID (
			const MISTRING& JobID
			);

		//! For scheduled jobs, check if should run at this time.  Must ReadJob first
		bool IsTimeToRun();

		//! Check if version job file was created for matches current version
		bool IsVersionMatch();

		//! Read existing job file then use GetVariable and GetObject
		ERRVALUE ReadJob (
			const FILEPATH& path			//!< Filepath of job file
			);

		//! Read existing job file then use GetVariable and GetObject.  Don't start status reporting
		ERRVALUE ReadJobNoStatus (
			const FILEPATH& path			//!< Filepath of job file
			);

		//! Set id of group job belongs to
		bool SetGroupID (
			const MISTRING& GroupID			//!< ID of group, call CreateGroup to generate new Group with GroupID
			);

		//! For scheduled jobs, set the next run time.  Must ReadJob first
		ERRVALUE SetNextTime ();

		//! Set job as "ready" to run.
		//! Used when job is run directly within process instead of writing to file.
		void SetReady (
			) { m_IsJobRead = true; }

		//! Write job file to default job directory
		ERRVALUE Write (
			bool held = false,				//!< True if need to create job as initially held
			MISTRING* jobid = 0,			//!< ID of job created
			bool bCreateAsTemp = false		//!< Create as temp group
			);

		//! Write job file to specific job directory
		ERRVALUE Write (
			const FILEPATH& directory,		//!< Directory to create job file in
			bool held = false,				//!< True if need to create job as initially held
			MISTRING* jobid = 0,			//!< ID of job created
			bool bCreateAsTemp = false		//!< Create as temp group
			);

		//! Write schedule job file
		ERRVALUE WriteScheduled (
			FILEPATH* path = 0				//!< Filepath of job file returned
			);

		//! Write xmldoc to current path
		ERRVALUE WriteDoc () {return m_doc.Write(m_path);}

	private:
		#ifndef GENERATING_DOXYGEN_OUTPUT
		bool m_IsJobRead;
		XMLDOC m_doc;
		XMLNODE *m_rootnode;
		XMLNODE *m_runparmsnode;
		MIJOBSTATUS *m_pStatusMgr;
		FILEPATH m_path;

		XMLNODE *FindValueNode (const char *name, const XMLNODE *node) const;
		#endif
	};

#endif		//  INC_MI32_MIJOB_H
