/**
 * \file rgnman.h <mi32/rgnman.h>
 * \brief Functions and structure for the region manager
 *
 * \if NODOC
 * $Id: rgnman.h_v 1.23 2009/07/02 18:01:36 bcolombe Exp $
 *
 * $Log: rgnman.h_v $
 * Revision 1.23  2009/07/02 18:01:36  bcolombe
 * Added GetRegionList
 *
 * Revision 1.22  2007/04/05 13:03:30  mju
 * No rvcgeninfo.
 *
 * Revision 1.21  2006/09/28 15:09:25  scowan
 * Added fwd decl.
 *
 * Revision 1.20  2005/09/19 17:13:06  scowan
 * Changed addRegion to take a descriptor.
 *
 * Revision 1.19  2005/01/17 20:27:44  mju
 * Include trans2dc hdr.
 *
 * Revision 1.18  2004/06/29 21:20:17  scowan
 * Changed tparm to be a trans 2d mapgen.
 *
 * Revision 1.17  2004/06/28 23:50:09  scowan
 * Changed function prototypes to use correct region class.
 *
 * Revision 1.16  2003/09/15 13:49:56  fileserver!dwilliss
 * Doxygen
 *
 * Revision 1.15  2000/07/13 18:52:50  sparsons
 * Genitor documentation.
 *
 * Revision 1.14  2000/07/12 21:37:38  mju
 * Include headers so compiles in isolation.
 *
 * \endif
**/

#ifndef	INC_MI32_RGNMAN_H
#define	INC_MI32_RGNMAN_H

#ifndef	INC_MI32_TRANS2DC_H
#include <mi32/trans2dc.h>
#endif

#if !defined(INC_MI32_XDEFNS_H) && defined(X_NATIVE)
#include <mi32/xdefns.h>
#endif

class REGION2D;

namespace RVC {
	class DESCRIPTOR;
	}

typedef void *RGNMANHANDLE;

struct MrgnManagerCallbackStruct {
	UINT32 reason;
	UINT32 ListNum;
	UINT32 NumRegions;
	TRANS2D_MAPGEN tparm;							//!<  Transparm to set for callback 
	void *vmxt;
	
	MrgnManagerCallbackStruct (
		) {
		reason = ListNum = NumRegions = 0;
		vmxt = 0;
		}
				
	};

#define MrgnCR_RegionAdded			0x00000001
#define MrgnCR_RegionRemoved		0x00000002
#define MrgnCR_SetTransParm		0x00000004
#define MrgnCR_SetTransparm		MrgnCR_SetTransParm
#define MrgnCR_Stop					0x00000008
#define MrgnCR_RegionSelected		0x00000010

#if defined(__cplusplus)
extern "C" {
#endif

//! Add callback to the region manager.
//!
//! Callback reasons:		
//!		MrgnCR_RegionAdded		Region has been added to the list
//!		MrgnCR_RegionRemoved		Region has been removed from the list
//!		MrgnCR_SetTransParm		Used to update the region to screen transparm
//!		MrgnCR_RegionSelected	Region has been selected in list
void MrgnManagerAddCallback (
	RGNMANHANDLE vpdp, 						//!< Region manager handle
	void (*CallbackFunc)(MrgnManagerCallbackStruct *, void *), 		//! Mcb callback function
	void *udata, 								//!< User data passed in callback
	UINT32 reason								//!< Callback reasons
	);

//! Add a region to the region manager.
int MrgnManagerAddRegion (
	RGNMANHANDLE vpdp, 						//!< Region manager handle
	const RVC::DESCRIPTOR& Descriptor,
	REGION2D& region							//!< The region to add
	);

//! Clear GraphicTool tool bound to region manager form.
void MrgnManagerFormClearTool (
	Widget rmform								//!< Form created by MrgnManagerFormCreate()
	);

//! Bind region manager form to ERegion GraphicTool.
void MrgnManagerFormBindTool (
	Widget rmform, 							//!< Form created by MrgnManagerFormCreate()
	void *vmxt									//!< Tool handle to bind to.
	);

#ifdef X_NATIVE

//! Create region manager form with region list and widgets.
//!
//! @return Form widget or NULL if error.
Widget MrgnManagerFormCreate (
	RGNMANHANDLE vpdp, 						//!< Region manager handle
	Widget parent, 							//!< Parent widget for form
	UINT32 flags, 								//!< For expansion
	Arg *formarg, 								//!< XT Args for form
	int formn									//!< Number of XT Args
	);

#endif

//! Update GraphicTool (ERegion) bound to form.
void MrgnManagerFormUpdateTool (
	Widget rmform								//!< Form created by MrgnManagerFormCreate()
	);

//! Free up the region manager toolkit.
int MrgnManagerFree (
	RGNMANHANDLE vpdp							//!< Region manager handle
	);

//! Get a region from the region manager.
//!
//! @return 1 if region returned, 0 if no region exists, < 0 error code.
int MrgnManagerGetRegion (
	RGNMANHANDLE vpdp, 						//!< Region manager handle
	INT32 ListNum, 							//!< Number of region to get, -1 for current region
	REGION2D& region							//!< Region structure RETURNED
	);

//! Get currently-active regions as objitemlist
void MrgnManagerGetRegionList (
	RGNMANHANDLE vpdp, 						//!< Region manager handle
	RVC::OBJITEMLIST& list
	);

//! Returns TRUE if a current region exists.
//!
//! @return 1 if region exists, 0 if no region exists.
int MrgnManagerHasRegion (
	RGNMANHANDLE vpdp							//!< Region manager handle
	);

//! Initialize region manager toolkit and return handle.
int MrgnManagerInit (
	RGNMANHANDLE *vpdp						//!< Region manager handle RETURNED
	);

//! Remove callbacks from the region manager.
//!
//! See MrgnManagerAddCallback() for a list of callback reasons.
void MrgnManagerRemoveCallback (
	RGNMANHANDLE vpdp, 						//!< Region manager handle
	void (*CallbackFunc)(MrgnManagerCallbackStruct *, void *), 	//! Mcb callback function
	void *udata, 								//!< User data passed in callback
	UINT32 reason								//!< Callback reasons
	);

//! Save the region manager.
int MrgnManagerSave (
	RGNMANHANDLE vpdp, 
	void *Widget, 
	UINT32 flags
	);

#if defined(__cplusplus)
}
#endif
#endif					//!<  #ifndef INC_MI32_RGNMAN_H 
