/**
 * \file rvc/hypspect.h 
 * \brief interface for the RVC::HYPERSPECTRAL class.
 *
 * \if NODOC
 * $Id: hypspect.h_v 1.29 2006/10/27 22:50:11 scowan Exp $
 *
 * $Log: hypspect.h_v $
 * Revision 1.29  2006/10/27 22:50:11  scowan
 * Made getCellType virtual.
 *
 * Revision 1.28  2006/07/28 13:31:42  mju
 * Use const in write.
 *
 * Revision 1.27  2006/07/26 22:59:34  scowan
 * Added readbandlossy methods.
 *
 * Revision 1.26  2006/06/27 15:04:18  scowan
 * Make method now fills in objitem with created object info.
 *
 * Revision 1.25  2006/06/26 16:36:12  scowan
 * Fixed shadow warning.
 *
 * Revision 1.24  2006/05/31 15:06:25  scowan
 * Added setCacheRect virtual method.
 *
 * Revision 1.23  2005/12/19 17:02:23  linux32build!dwilliss
 * Credentials can now be made mandatory by setting a #define
 *
 * Revision 1.22  2005/12/09 17:29:34  scowan
 * Updated to use credentials.
 *
 * Revision 1.21  2005/10/07 16:47:30  scowan
 * Added write line methods.
 *
 * Revision 1.20  2005/10/07 16:40:21  scowan
 * Updated to new system.
 *
 * Revision 1.19  2004/08/31 17:46:27  scowan
 *  Support cache size virtual methods.
 *
 * Revision 1.18  2004/02/24 15:55:48  scowan
 * Added x, y, and tile size,
 *
 * Revision 1.17  2003/11/21 22:17:06  scowan
 * Support new Capabilities system.
 *
 * Revision 1.16  2003/10/07 17:32:30  scowan
 * Object virtual method changes.
 *
 * Revision 1.15  2003/09/30 16:52:06  dwilliss
 * Doxygen
 *
 * Revision 1.14  2003/09/25 21:20:04  dwilliss
 * Fixed doxygen comment
 *
 * Revision 1.13  2003/09/15 23:17:44  scowan
 * Support new set null value virtual method.
 *
 * Revision 1.12  2003/09/15 13:49:45  fileserver!dwilliss
 * Doxygen
 *
 * Revision 1.11  2003/08/27 21:28:30  scowan
 * I did something.
 *
 * Revision 1.10  2003/06/27 20:40:05  scowan
 * Added getting methods.
 *
 * Revision 1.9  2003/05/07 22:20:09  scowan
 * Changed virtual method format.
 *
 * Revision 1.8  2002/09/05 21:31:38  scowan
 * Change include file name.
 *
 * Revision 1.7  2002/08/23 13:27:58  mju
 * nc.
 *
 * Revision 1.6  2002/04/26 22:44:39  scowan
 * Added more virtual methods.
 *
 * Revision 1.5  2001/11/02 17:15:34  mju
 * Use CELLTYPE instead of DATATYPE/numbits.
 *
 * Revision 1.4  2001/10/25 21:29:31  scowan
 * Nothing.
 *
 * Revision 1.3  2001/06/14 15:04:37  scowan
 * Added to RVC namespace.
 *
 * Revision 1.2  2001/06/13 16:11:41  scowan
 * Changed objtype enum.
 *
 * Revision 1.1  2001/05/25 22:28:01  scowan
 * Initial revision
 *
 * \endif
 **/

//!	\class RVC::HYPERSPECTRAL rvc/hypspect.h
//!	\brief Manages RVC::OBJTYPE_Hyperspectral RVC objects.  
//!	
//!	The interface currently wraps the MfHyperspectral functions.
//!	
//!	RVC Object Types that this class supports: RVC::OBJTYPE_Hyperspectral
//!
//!	Default RVC Object Type for this class: RVC::OBJTYPE_Hyperspectral
//!
//!	Valid parents for this class:
//!		- RVC::OBJTYPE_File
//!		- RVC::OBJTYPE_Folder

#ifndef INC_RVC_HYPSPECT_H
#define INC_RVC_HYPSPECT_H

#ifndef INC_RVC_IMAGE_H
	#include <rvc/image.h>
#endif

#ifndef INC_MI32_DATETIME_H
	#include <mi32/datetime.h>
#endif

namespace RVC {

#ifndef GENERATING_DOXYGEN_OUTPUT
class HYPERSPECTRALHANDLE;
#endif // GENERATING_DOXYGEN_OUTPUT

class HYPERSPECTRAL : public RVC::IMAGE {
	public:
		#ifndef GENERATING_DOXYGEN_OUTPUT
		class MAKEPARMS;	
		#endif // GENERATING_DOXYGEN_OUTPUT
	
		//! Default constructor
		HYPERSPECTRAL (
			);

		//! Copy constructor
		HYPERSPECTRAL (
			const RVC::HYPERSPECTRAL& rhs
			);

		//! Destructor
	   virtual ~HYPERSPECTRAL (
			);

		//! Assignment
		RVC::HYPERSPECTRAL& operator= (
			const RVC::HYPERSPECTRAL& rhs
			);

		//! Obtain number of primary vectors in a hyperspectral image
		//! @return number of primary vectors in the hyperspectral image
		INT32 GetNumPrimaryVectors (
			) const {return (m_NumVectors); }

		//! Obtain number of columns in a hyperspectral image tile
		//! @return number of columns in the hyperspectral image tile
		INT32 GetNumTileColumns (
			) const {return (m_TileColumns);}

		//! Obtain number of lines in a hyperspectral image tile
		//! @return number of lines in the hyperspectral image tile
		INT32 GetNumTileLines (
			) const {return (m_TileLines);}

		//! Obtain number of tiles in a raster row
		//! @return number of row tiles
		INT32 GetNumXTiles (
			) const {return (m_XTiles);}

		//! Obtain number of tiles in a raster column
		//! @return number of column tiles
		INT32 GetNumYTiles (
			) const {return (m_YTiles);}

		//! Obtain number of bytes in a raster tile
		//! @return size of raster tile in bytes
		INT32 GetTileSizeBytes (
			) const {return (m_TileSize);}

		//! Create a hyperspectral object
		ERRVALUE Make (
	    	const RVC::OBJECT& parent,					//!< Parent object to create raster under
			const RVC::DESCRIPTOR& Descriptor,
			MAKEPARMS& MakeParms,							//!< Parameters needed to create raster, cannot be changed after creation
			const CREDENTIALS* Credentials RVC_DEFAULT_CREDENTIALS				//!< User credentials passed in, NULL for default behavior
			);

		//! Create a hyperspectral object
		ERRVALUE Make (
	    	RVC::OBJITEM& ObjItem,						//!< Parent object and DESCRIPTOR of object to create, new location RETURNED
			MAKEPARMS& MakeParms,						//!< Parameters needed to create raster, cannot be changed after creation
			const char* Source = 0,						//!< Source of raster creation, default is GetAppName()
			const CREDENTIALS* Credentials RVC_DEFAULT_CREDENTIALS				//!< User credentials passed in, NULL for default behavior
			);

		//! Read a line of raster cells up to a certain length
		ERRVALUE ReadBandLossy (
			INT32 Band,
			INT32 Line,									//!< Raster line to read
			INT32 StartColumn,						//!< Raster column to start reading on
			void *data,									//!< Buffer to read into
			INT32 Length = 1,							//!< Number of columns to read
			CONVMODE Convert = CONVMODE_None		//!< Data conversion mode
			) const;

		//! Read a line of raster cells up to a certain length, converting to signed 8 bit
		ERRVALUE ReadBandLossy (								//! Read a raster line, signed 8 bit
			INT32 Band,									//!< Image band number to read
			INT32 Line,									//!< Raster line to read
			INT32 StartColumn,							//!< Raster column to start reading on
			INT8 *data,									//!< Buffer to read into
			INT32 Length = 1							//!< Number of columns to read
			) const { return (ReadBandLossy(Band, Line, StartColumn, data, Length, CONVMODE_INT8)); }

		//! Read a line of raster cells up to a certain length, converting to unsigned 8 bit
		ERRVALUE ReadBandLossy (								//! Read a raster line, unsigned 8 bit
			INT32 Band,									//!< Image band number to read
			INT32 Line,									//!< Raster line to read
			INT32 StartColumn,							//!< Raster column to start reading on
			UINT8 *data, 								//!< Buffer to read into
			INT32 Length = 1							//!< Number of columns to read
			) const { return (ReadBandLossy(Band, Line, StartColumn, data, Length, CONVMODE_INT8)); }

		//! Read a line of raster cells up to a certain length, converting to signed 16 bit
		ERRVALUE ReadBandLossy (								//! Read a raster line, signed 16 bit
			INT32 Band,									//!< Image band number to read
			INT32 Line,									//!< Raster line to read
			INT32 StartColumn,							//!< Raster column to start reading on
			INT16 *data, 								//!< Buffer to read into
			INT32 Length = 1							//!< Number of columns to read
			) const { return (ReadBandLossy(Band, Line, StartColumn, data, Length, CONVMODE_INT16)); }

		//! Read a line of raster cells up to a certain length, converting to unsigned 16 bit
		ERRVALUE ReadBandLossy (								//! Read a raster line, unsigned 16 bit
			INT32 Band,									//!< Image band number to read
			INT32 Line,									//!< Raster line to read
			INT32 StartColumn,							//!< Raster column to start reading on
			UINT16 *data,								//!< Buffer to read into
			INT32 Length = 1							//!< Number of columns to read
			) const { return (ReadBandLossy(Band, Line, StartColumn, data, Length, CONVMODE_INT16)); }

		//! Read a line of raster cells up to a certain length, converting to signed 32 bit
		ERRVALUE ReadBandLossy (								//! Read a raster line, signed 32 bit
			INT32 Band,									//!< Image band number to read
			INT32 Line,									//!< Raster line to read
			INT32 StartColumn,							//!< Raster column to start reading on
			INT32 *data,								//!< Buffer to read into
			INT32 Length = 1							//!< Number of columns to read
			) const { return (ReadBandLossy(Band, Line, StartColumn, data, Length, CONVMODE_INT32)); }

		//! Read a line of raster cells up to a certain length, converting to unsigned 32 bit
		ERRVALUE ReadBandLossy (								//! Read a raster line, unsigned 32 bit
			INT32 Band,									//!< Image band number to read
			INT32 Line,									//!< Raster line to read
			INT32 StartColumn,							//!< Raster column to start reading on
			UINT32 *data,								//!< Buffer to read into
			INT32 Length = 1							//!< Number of columns to read
			) const { return (ReadBandLossy(Band, Line, StartColumn, data, Length, CONVMODE_INT32)); }

		//! Read a line of raster cells up to a certain length, converting to 32 bit float
		ERRVALUE ReadBandLossy (								//! Read a raster line, 4 byte floats
			INT32 Band,									//!< Image band number to read
			INT32 Line,									//!< Raster line to read
			INT32 StartColumn,							//!< Raster column to start reading on
			float *data,								//!< Buffer to read into
			INT32 Length = 1							//!< Number of columns to read
			) const { return (ReadBandLossy(Band, Line, StartColumn, data, Length, CONVMODE_Float)); }

		//! Read a line of raster cells up to a certain length, converting to 64 bit double
		ERRVALUE ReadBandLossy (								//! Read a raster line, 8 byte doubles
			INT32 Band,									//!< Image band number to read
			INT32 Line,									//!< Raster line to read
			INT32 StartColumn,							//!< Raster column to start reading on
			double *data, 								//!< Buffer to read into
			INT32 Length = 1							//!< Number of columns to read
			) const { return (ReadBandLossy(Band, Line, StartColumn, data, Length, CONVMODE_Double)); }

		ERRVALUE WriteAverageLine (
			INT32 Line,
			INT32 StartColumn,
			const void* data,
			INT32 Length,
			CONVMODE Convert = CONVMODE_None			//!< Data conversion mode
			);
	
		ERRVALUE WriteBandLine (
			INT32 Line,
			INT32 StartColumn,
			const void* data,
			INT32 Length,
			CONVMODE Convert = CONVMODE_None			//!< Data conversion mode
			);
	
		ERRVALUE WriteClassLine (
			INT32 Line,
			INT32 StartColumn,
			const void* data,
			INT32 Length,
			CONVMODE Convert = CONVMODE_None			//!< Data conversion mode
			);
	
		ERRVALUE WriteResidualLine (
			INT32 Band,
			INT32 Line,
			INT32 StartColumn,
			const void* data,
			INT32 Length,
			CONVMODE Convert = CONVMODE_None			//!< Data conversion mode
			);
			
	private:
		#ifndef GENERATING_DOXYGEN_OUTPUT

		//! Implementation of RVC::OBJECT virtual methods
		virtual ERRVALUE v_CloseObject ();
		virtual RVC::OBJTYPE v_GetDftObjectType () const;
		virtual bool v_IsObjectOpen () const;					
		virtual ERRVALUE v_OpenObject (OPENMODE OpenMode, MDLGPARENT parent, const CREDENTIALS* Credentials);
				
		//! Implementation of RVC::GEOSPATIAL virtual methods
		virtual void v_GetExtents (DRECT3D &extents) const;
		virtual void v_GetScale (double& xscale, double& yscale) const;
		virtual double v_GetZScale () const;
		virtual double v_GetZOffset () const;
		virtual bool v_HasCapability (int Capability) const;
		virtual void v_SetScale (double xscale, double yscale);
		virtual void v_SetZScale (double zscale);
		virtual void v_SetZOffset (double zoffset);
			
		//! Implementation of RVC::IMAGE virtual methods
		virtual ERRVALUE v_GetBandParms (INT32 BandNum, BANDPARMS& BandParms) const;
		virtual CELLTYPE v_GetCellType () const;
		virtual double v_GetColumnScale () const;
		virtual double v_GetLineScale () const;
		virtual ERRVALUE v_GetNullValue (INT32 BandNum, NULLVALUE& NullValue) const;
		virtual INT32 v_GetNumBands () const;
		virtual INT32 v_GetNumColumns () const;
		virtual INT32 v_GetNumLines () const;
		virtual bool v_HasNullValue (INT32 BandNum) const;
		virtual ERRVALUE v_ReadBand (INT32 Band, INT32 Line, INT32 StartColumn, void *data, INT32 Length, CONVMODE Convert) const;
		virtual ERRVALUE v_ReadColumnBand (INT32 Band, INT32 Line, INT32 StartColumn, void *data, INT32 Length, CONVMODE Convert) const;
		virtual ERRVALUE v_SetCacheRect (const LRECT2D& Rect);
		virtual ERRVALUE v_SetCacheSize (INT32 NumTiles);
		virtual ERRVALUE v_SetCacheSize (CACHEMODE CacheFlags);
		virtual ERRVALUE v_SetNullValue (INT32 BandNum, const NULLVALUE* NullValue);
		virtual ERRVALUE v_WriteBand (INT32 Band, INT32 Line, INT32 StartColumn, const void *data, INT32 Length, CONVMODE Convert);
		
		HYPERSPECTRALHANDLE* m_HyperSpectralID;
		INT32 m_NumLines;
		INT32 m_NumColumns;
		INT32 m_NumBands;
		INT32 m_NumVectors;
		CELLTYPE m_CellType;
		INT32 m_TileLines;				//!< Number of lines per tile
		INT32 m_TileColumns;			//!< Number of columns per tile
		INT32 m_XTiles;					//!< Number of XTiles
		INT32 m_YTiles;					//!< Number of YTiles
		INT32 m_TileSize;				//!< Size of tile in bytes
		#endif // GENERATING_DOXYGEN_OUTPUT
	};


class HYPERSPECTRAL::MAKEPARMS {
	public:
		//! Constructor, initialize with critical Hyperspectral creation parameters
		MAKEPARMS (
			INT32 NumLines,
			INT32 NumColumns,
			INT32 NumBands,					//!<  Number of bands in the hyperspectral image 
			INT32 NumVectors,					//!<  Number of primary vectors 
			IMAGE::CELLTYPE CellType
			) :
			m_NumLines(NumLines),
			m_NumColumns(NumColumns),
			m_CellType(CellType),
			m_TileLines(0),
			m_TileColumns(0),
			m_IsLossy(false),
			m_LineScale(0.0),
			m_ColumnScale(0.0),
			m_NumBands(NumBands),
			m_NumVectors(NumVectors)
			{ }

		//! Template constructor
		MAKEPARMS (
			const HYPERSPECTRAL& Template,				//!< Hyperspectral template to determine NumLines, NumColumns, NumBands, NumVdectors, and scaling
			IMAGE::CELLTYPE CellType = CELLTYPE_Invalid	//!< Optional cell type, default uses type from template
			) :
			m_NumLines(Template.GetNumLines()),
			m_NumColumns(Template.GetNumColumns()),
			m_CellType((CellType!=CELLTYPE_Invalid)?CellType:Template.GetCellType()),
			m_TileLines(Template.GetNumTileLines()),
			m_TileColumns(Template.GetNumTileColumns()),
			m_IsLossy(false),
			m_LineScale(0.0),
			m_ColumnScale(0.0),
			m_NumBands(Template.GetNumBands()),
			m_NumVectors(Template.GetNumPrimaryVectors())
			{
			Template.GetScale(m_ColumnScale, m_LineScale);
			}
	
		//! Obtain image cell type.
		//! @return RVC::IMAGE::CELLTYPE for the hyperspectral.
		IMAGE::CELLTYPE GetCellType (
			) const {return (m_CellType);}

		//! Obtain collection date for the hyperspectral object
		const DATETIME& GetCollectionDate (
			) const { return (m_CollectionDate); }

		//! Obtain line scale value for the hyperspectral object
		//! @return line scale value
		double GetColumnScale (
			) const {return (m_ColumnScale); }
			
		//! Obtain datatype based on hyperspectral creation parameters
		//! @return RVC::IMAGE::DATATYPE of the hyperspectral
		IMAGE::DATATYPE GetDataType (
			) const {return (IMAGE::GetDataTypeFromCellType(m_CellType));}

		//! Obtain default hyperspectral tile size based on creation parameters
		//! @return default tile size
		INT32 GetDftTileSize (
			) const;

		//! Obtain line scale value for the hyperspectral object
		//! @return line scale value
		double GetLineScale (
			) const {return (m_LineScale); }

		//! Obtain number of bands in the hyperspectral object
		//! @return number of bands
		INT32 GetNumBands (
			) const {return (m_NumBands); }

		//! Obtain number of bits in a hyperspectral cell
		//! @return number of hyperspectral bits
		UINT16 GetNumBits (
			) const {return (IMAGE::GetNumBitsFromCellType(m_CellType));}

		//! Obtain number of columns in a hyperspectral
		//! @return number of hyperspectral columns
		INT32 GetNumColumns (
			) const {return (m_NumColumns);}

		//! Obtain number of lines in a hyperspectral
		//! @return number of hyperspectral lines
		INT32 GetNumLines (
			) const {return (m_NumLines);}

		//! Obtain number of primary vectors in the hyperspectral object
		//! @return number of primary vectors
		INT32 GetNumPrimaryVectors (
			) const {return (m_NumVectors); }

		//! Obtain the sensor name for this hyperspectral object
		//! @return Sensor Name
		const OBJECTNAME& GetSensorName (
			) const {return (m_SensorName);}

		//! Set hyperspectral tile size, only if you know what you are doing!
		void GetTileSize (
			INT32& TileLines,
			INT32& TileColumns
			) const { TileLines = m_TileLines; TileColumns = m_TileColumns; }
			
		//! Determiine if the hyperspectral object is lossy compressed or not
		//! @return "True'if the hyperspectral object is lossy compressed
		bool IsLossy (
			) const { return (m_IsLossy); }

		//! Set image cell type.
		void SetCellType (
			CELLTYPE CellType
			) {m_CellType = CellType;}

		//! Set collection date for the hyperspectral object
		void SetCollectionDate (
			const DATETIME& Date
			) { m_CollectionDate = Date; }

		//! Set column scale value for the hyperspectral object
		void SetColumnScale (
			double scale
			) { m_ColumnScale = scale; }

		//! Set the hyperspectral object to a lossy compression or not
		void SetIsLossy (
			bool bIsLossy
			) { m_IsLossy = bIsLossy; }

		//! Set line scale value for the hyperspectral object
		void SetLineScale (
			double scale
			) { m_LineScale = scale; }

		//! Set the number of bands in the hyperspectral object
		void SetNumBands (
			INT32 NumBands
			) { m_NumBands = NumBands; }

		//! Set number of columns in a hyperspectral
		void SetNumColumns (
			INT32 NumColumns
			) {m_NumColumns = NumColumns;}

		//! Set number of lines in a hyperspectral
		void SetNumLines (
			INT32 NumLines
			) {m_NumLines = NumLines;}

		//! Set the number of primary vectors in the hyperspectral object
		void SetNumPrimaryVectors (
			INT32 NumVectors
			) { m_NumVectors = NumVectors; }

		//! Obtain the sensor name for this hyperspectral object
		//! @return Sensor Name
		void SetSensorName (
			const OBJECTNAME& name
			) { m_SensorName = name; }

		//! Set hyperspectral tile size, only if you know what you are doing!
		void SetTileSize (
			INT32 TileLines,
			INT32 TileColumns
			) { m_TileLines = TileLines; m_TileColumns = TileColumns; }
	
	private:
		#ifndef GENERATING_DOXYGEN_OUTPUT

		INT32 m_NumLines;					//!< Number of lines in hyperspectral
		INT32 m_NumColumns;				//!< Number of columns in hyperspectral
		CELLTYPE m_CellType;				//!< Image cell type
		INT32 m_TileLines;				//!< Number of lines per tile
		INT32 m_TileColumns;				//!< Number of columns per tile
		bool m_IsLossy;
		double m_LineScale;				//!<  Line scale in units for given type 
		double m_ColumnScale;			//!<  Column scale in units for given type 
		INT32 m_NumBands;					//!<  Number of bands in the hyperspectral image 
		INT32 m_NumVectors;				//!<  Number of primary vectors 
		RVC::OBJECTNAME m_SensorName;
		DATETIME m_CollectionDate;		//!<  Date hyperspectral set data was aquired 

		friend class HYPERSPECTRAL;
		#endif // GENERATING_DOXYGEN_OUTPUT
	};

}		//! End of RVC namespace

#endif 	//!< INC_RVC_HYPSPECT_H
