/**
 * \file rvc/spectrallibrary.h 
 * \brief interface for the RVC::SPECTRALLIBRARY class.
 *
 * \if NODOC
 * $Id: spectrallibrary.h_v 1.5 2011/12/23 14:07:13 mju Exp $
 *
 * $Log: spectrallibrary.h_v $
 * Revision 1.5  2011/12/23 14:07:13  mju
 * Remove redundant hdrs.
 *
 * Revision 1.3  2006/07/27 16:31:54  scowan
 * Changed names of record and channel structs.
 *
 * Revision 1.1  2006/06/23 15:30:07  scowan
 * Initial revision
 * \endif
**/

#ifndef INC_RVC_SPECTRALLIBRARY_H
#define INC_RVC_SPECTRALLIBRARY_H

#ifndef INC_RVC_OBJECT_H
	#include <rvc/object.h>
#endif


namespace RVC {
#ifndef GENERATING_DOXYGEN_OUTPUT
	class SPECTRALLIBRARYHANDLE;
#endif // GENERATING_DOXYGEN_OUTPUT

//! Manages RVC::OBJTYPE_Spectral RVC objects.  
//!	
//!	RVC Object Types that this class supports: RVC::OBJTYPE_Spectral
//!
//!	Valid parents for this class:
//!		RVC::OBJTYPE_File
//!		RVC::OBJTYPE_Folder
//!
//! Spectral library is a collection of hyperspectral records
//! for number of different materials, minerals plants and etc.
//! Hyperspectral record contains the calibrated reflectance
//! (or measured radiance) detected at multiple wavelength channels.
//! Each wavelength channel has a spectral definition that consists
//! of wavelength, bandwidth and error value associated with a
//! specific measurement.
//! "Channel" is a USGS term for describing elementary measurement 
//! at certain wavelength.
class SPECTRALLIBRARY : public RVC::OBJECT {
	public:
		#ifndef GENERATING_DOXYGEN_OUTPUT
		struct RECORD;
		struct CHANNEL;
		#endif // GENERATING_DOXYGEN_OUTPUT

		//! Default constructor
		SPECTRALLIBRARY (
			);

		//! Copy constructor
		SPECTRALLIBRARY (
			const RVC::SPECTRALLIBRARY& obj
			);

		//! Destructor
		virtual ~SPECTRALLIBRARY (
			);

		//! Assignment
		RVC::SPECTRALLIBRARY& operator= (
			const RVC::SPECTRALLIBRARY& rhs
			);

		//! Add a record to the rvc spectral library object, do not use WriteRecord() since multiple people 
		//! can be trying to add a record at the same time
		//! @return Index of new record of error if < 0
		INT32 AddRecord (
			const RECORD& record								//!< Record to add
			);

		//! Delete given record
		//!
		//! This method deletes a record by swapping the last record with 'ShapeNum', 
		//! then it resizes the number of records by -1.
		ERRVALUE DeleteRecord (
			INT32 RecordNum									//!< Shape number to delete
			);
			
		INT32 GetMaxNumChannels (
			);
			
		double GetMaxWavelength (
			);
			
		INT32 GetMinNumChannels (
			);
			
		double GetMinWavelength (
			);
			
		//! Obtain the number of records in a rvc spectral library object
		//! @return number of records
		INT32 GetNumRecords (
			);

		//! Create a new rvc spectrallibrary object under a specified parent rvc object with name, description, and source
		ERRVALUE Make (
			const RVC::OBJECT& parent,		//!< Parent object to create style object under
			const RVC::DESCRIPTOR& Descriptor,
			const CREDENTIALS* Credentials RVC_DEFAULT_CREDENTIALS				//!< User credentials passed in, NULL for default behavior
			);

		//! Create a new rvc style object under a specified parent rvc object with name, description, and source
		ERRVALUE Make (
			RVC::OBJITEM& ObjItem,	//!< Name, Desc of object to create, Created object location RETURNED
#ifdef RVC_CREDENTIALS_MANDATORY
			const char* Source,			//!< Source of new style object, default is application name
			const CREDENTIALS* Credentials //!< User credentials passed in, NULL for default behavior
#else
			const char* Source = 0,			//!< Source of new style object, default is application name
			const CREDENTIALS* Credentials = 0				//!< User credentials passed in, NULL for default behavior
#endif
			);

		//! Read a rvc spectrallibrary object record element
		ERRVALUE ReadRecord (
			INT32 RecordNum,									//!< The record index to read
			RECORD& record							//!< Record to read
			);

		//! Read a rvc spectrallibrary object record channels
		ERRVALUE ReadRecordChannels (
			INT32 RecordNum,									//!< The record index to read
			SIMPLE_ARRAY<CHANNEL>& Channels	//!< Channel array RETURNED
			);

		//! Read a rvc spectrallibrary object record element
		ERRVALUE WriteRecord (
			INT32 RecordNum,									//!< The record index to read
			RECORD& record							//!< Record to read
			);

		//! Write a rvc spectrallibrary object record channels
		ERRVALUE WriteRecordChannels (
			INT32 RecordNum,											//!< The record index to write
			const SIMPLE_ARRAY<CHANNEL>& Channels	//!< Channel array to write
			);

	private:
		#ifndef GENERATING_DOXYGEN_OUTPUT

		//! Implementation of RVC::OBJECT virtual methods
		virtual ERRVALUE v_CloseObject ();
		virtual OBJTYPE v_GetDftObjectType () const;
		virtual bool v_IsObjectOpen () const;
		virtual ERRVALUE v_OpenObject (OPENMODE OpenFlags, MDLGPARENT parent, const CREDENTIALS* Credentials);

		SPECTRALLIBRARYHANDLE* m_SpectralLibraryHandle;
		#endif // GENERATING_DOXYGEN_OUTPUT
	};


//! Spectral channel header definition
struct SPECTRALLIBRARY::CHANNEL {
	FLOAT wavelength;							//!< Micrometers
	FLOAT bandwidth;							//!< Micrometers
	FLOAT error;								//!< Micrometers
	FLOAT value;								//!< In 0...1 range
	};

//! Measurement value and definition for one spectral channel.
#define SPECTRUM_TITLE_SIZE			66
#define SPECTRUM_HISTORY_SIZE			66
#define SPECTRUM_USERNAME_SIZE		16

//! Spectral record header definition.
struct SPECTRALLIBRARY::RECORD {
	FIXEDSTRING<SPECTRUM_TITLE_SIZE> Title;	//!< Material name 
	FIXEDSTRING<SPECTRUM_HISTORY_SIZE> History;//!< History or description of the conditions when record was generated 
	FIXEDSTRING<SPECTRUM_USERNAME_SIZE> Username;//!< USGS username who created the spectrum 
	RVCDATETIME ProcDate;	  				//!< USGS field 
	RVCDATETIME ProcDataTime;				//!< USGS field 
	RVCDATETIME StartDataTime;				//!< USGS field 
	DOUBLE RightAscention;					//!< USGS: right ascension coordinates of an astronomical object or longitude on surface 
	DOUBLE DeclinationCoord;				//!< USGS: declination coordinates of an astronomical object or latitude on surface 
	DOUBLE IncidentAngle;					//!< USGS: the angle of incidence of illuminating radiation 
	DOUBLE EmissionAngle;					//!< USGS Manual History field 
	DOUBLE PhaseAngle;						//!< USGS Manual History field 
	DOUBLE SampleBeam;						//!< USGS Manual History field 
	DOUBLE OneScanTime;						//!< USGS Manual History field 
	DOUBLE IntegrationTime;					//!< USGS Manual History field 
	DOUBLE Temperature;						//!< USGS Manual History field: object temperature at the moment of the observation 
	DOUBLE AthmosphericThickness;			//!< USGS: the equivalent athmospheric thickness through which the observation was obtained 
	DOUBLE MinWavelength;					//!< Min. wavelength for this record 
	DOUBLE MaxWavelength;					//!< Max. wavelength for this record 
	INT32 IndepSpectRuns;					//!< USGS: number of independent spectral runs which were added to make spectrum 
	INT32 WeightNumRuns;						//!< USGS: the number of independent spectral runs which were summed or averaged to make this spectrum 
	INT32 NormChannels[2];					//!< USGS: channel numbers which define the band normalization 
	
	RECORD (
		) { memset(this, 0, sizeof(*this)); }
		
	};

}		// End of RVC namespace

#endif			// INC_RVC_SPECTRALLIBRARY_H
