/**
 * \file rvc/stylepatternbitmap.h 
 * \brief interface for the RVC::STYLEPATTERN_BITMAP class.
 *
 * \if NODOC
 * $Id: stylepatternbitmap.h_v 1.7 2007/10/15 15:51:45 dwilliss Exp $
 *
 * $Log: stylepatternbitmap.h_v $
 * Revision 1.7  2007/10/15 15:51:45  dwilliss
 * Added method to import from any SPATMOD::IMAGE::STAGE
 *
 * Revision 1.6  2007/10/11 21:10:39  dwilliss
 * Added optional parameter to Create method
 *
 * Revision 1.5  2007/10/02 21:04:41  dwilliss
 * Allow for the possibility of RGBA bitmap patterns
 *
 * Revision 1.4  2007/04/04 17:49:21  scowan
 * Added virtual methods.
 *
 * Revision 1.3  2006/09/21 14:20:04  scowan
 * Added pattern type to the base.
 *
 * Revision 1.2  2006/09/20 16:54:02  dwilliss
 * Added ctor from raw buffer
 *
 * Revision 1.1  2006/09/18 17:54:22  scowan
 * Initial revision
 *
 * \endif
**/

#ifndef INC_RVC_STYLEPATTERNBITMAP_H
#define INC_RVC_STYLEPATTERNBITMAP_H

#ifndef INC_RVC_STYLEPATTERNBASE_H
	#include <rvc/stylepatternbase.h>
#endif

namespace SPATMOD {
	namespace IMAGE {
		class STAGE;
		}
	}

namespace RVC {

//! Bitmap pattern class.	
class STYLEPATTERN_BITMAP : public STYLEPATTERN_BASE {
	public:

		//! Default constructor.
		STYLEPATTERN_BITMAP (
			) : STYLEPATTERN_BASE(PATTERNTYPE_Bitmap, sizeof(PATTERNHEADER_BITMAP)) {}

		//! Constructor from raw pattern data
		STYLEPATTERN_BITMAP (
			const void* buf,
			INT32 NumBytes
			) : STYLEPATTERN_BASE(PATTERNTYPE_Bitmap, sizeof(PATTERNHEADER_BITMAP)) {
			SetPattern(buf, NumBytes, sizeof(PATTERNHEADER_BITMAP));
			}

		//! Copy constructor.
		STYLEPATTERN_BITMAP (
			const STYLEPATTERN_BITMAP& rhs
			) : STYLEPATTERN_BASE(rhs) {}

		//! Destructor.
		virtual ~STYLEPATTERN_BITMAP (
			) { }

		//! Assignment.
		STYLEPATTERN_BITMAP& operator= (
			const STYLEPATTERN_BITMAP& rhs
			) { if (this != &rhs) { STYLEPATTERN_BASE::operator=(rhs); } return (*this); }
			
		//! Create a bitmap pattern. 
		//! By default, bitmap patterns are 8-bit colormapped patterns.  If bRGBA
		//! is true, then the pattern will be created as an RGBA pattern
		ERRVALUE Create (
			INT16 xsize,
			INT16 ysize,
			bool bRGBA = false
			);
			

		//! Get the value of a given cell.
		//! Note: this assumes an 8-bit colormapped pattern
		UINT8 GetCell (
			INT16 x,
			INT16 y
			) const { return (GetFirstElemPtr()[y*GetSizeX()+x]); }

		//! Get the value of a given cell.
		//! Note: this assumes an RGBA pattern and will not work for 8-bit patterns
		void GetCell (
			INT16 x,
			INT16 y,
			COLOR& color
			) const { color.SetByColorSpace(COLORSPACE_R8_G8_B8_A8, GetFirstElemPtr() + (y*GetSizeX()+x)*4); }

		//! Get the data for a given row.
		//! If the pattern is RGBA, this will return a pointer to RGBA data,
		//! otherwise it will return a pointer to 8-bit colormapped data
		const UINT8* GetRowData (
			INT16 y
			) const { return (GetFirstElemPtr() + y*GetSizeX()*(IsRGBA()?4:1)); }

		//! Get the data for a given row.
		//! If the pattern is RGBA, this will return a pointer to RGBA data,
		//! otherwise it will return a pointer to 8-bit colormapped data
		UINT8* GetRowData (
			INT16 y
			) { return (GetFirstElemPtr() + y*GetSizeX()*(IsRGBA()?4:1)); }


		INT16 GetHotSpotX (
			) const { return (reinterpret_cast<const PATTERNHEADER_BITMAP*>(m_PattBuff)->xhot); }

		INT16 GetHotSpotY (
			) const { return (reinterpret_cast<const PATTERNHEADER_BITMAP*>(m_PattBuff)->yhot); }

		INT16 GetIntervalX (
			) const { return (reinterpret_cast<const PATTERNHEADER_BITMAP*>(m_PattBuff)->xspace); }

		INT16 GetIntervalY (
			) const { return (reinterpret_cast<const PATTERNHEADER_BITMAP*>(m_PattBuff)->yspace); }

		INT16 GetSizeX (
			) const { return (reinterpret_cast<const PATTERNHEADER_BITMAP*>(m_PattBuff)->xsize); }

		INT16 GetSizeY (
			) const { return (reinterpret_cast<const PATTERNHEADER_BITMAP*>(m_PattBuff)->ysize); }

		//! Import a bitmap from any SPATMOD::IMAGE::STAGE.
		//! The Source should be Initalize()ed first.
		//! The Source may not be larger than 32K by 32K.
		//! This method will not set the name of the STYLEPATTERN_BITMAP.  You
		//!   should do that based on the filename or user-entered name.
		ERRVALUE Import (
			SPATMOD::IMAGE::STAGE& Source
			);

		//! Determine if this bitmap pattern is RGBA or 8-bit colormapped
		//! If it's RGBA, GetNumColors() will return -1.
		bool IsRGBA (
			) const { return (GetNumColors() == -1); }
		
		//! Set the pixel value of a given cell
		//! Note: this assumes an 8-bit colormapped pattern
		void SetCell (
			INT16 x,
			INT16 y,
			UINT8 value
			) { GetFirstElemPtr()[y*GetSizeX()+x] = value; }

		//! Set the pixel value of a given cell
		//! Note: this assumes an RGBA pattern and will not work for 8-bit patterns
		void SetCell (
			INT16 x,
			INT16 y,
			const COLOR& value
			) { value.GetByColorSpace(COLORSPACE_R8_G8_B8_A8, GetFirstElemPtr() + (y*GetSizeX()+x)*4); }

		void SetHotSpotX (
			INT16 hotspotx
			) { reinterpret_cast<PATTERNHEADER_BITMAP*>(m_PattBuff)->xhot = hotspotx; }

		void SetHotSpotY (
			INT16 hotspoty
			) { reinterpret_cast<PATTERNHEADER_BITMAP*>(m_PattBuff)->yhot = hotspoty; }

		void SetIntervalX (
			INT16 intervalx
			) { reinterpret_cast<PATTERNHEADER_BITMAP*>(m_PattBuff)->xspace = intervalx; }

		void SetIntervalY (
			INT16 intervaly
			) { reinterpret_cast<PATTERNHEADER_BITMAP*>(m_PattBuff)->yspace = intervaly; }

	private:
		#ifndef GENERATING_DOXYGEN_OUTPUT
		//! Bitmap pattern header.
		struct PATTERNHEADER_BITMAP : public STYLEPATTERN_BASE::PATTERNHEADER {
			INT16 xsize;								//!< X size
			INT16 ysize;								//!< Y size
			INT16 xhot;									//!< X hot					
			INT16 yhot;									//!<	Y hot
			INT16 xspace;								//!< X space
			INT16 yspace;								//!< Y space
			};

		virtual bool v_IsEmpty () const;
		#endif // GENERATING_DOXYGEN_OUTPUT
	};

}		//! End of RVC namespace

#endif 	//!< INC_RVC_STYLEPATTERNITMAP_H
	
