/**
 * \file spatmod/imagefilterresample.h
 * \brief Spatial Modelling (SPATMOD) IMAGE::FILTER_RESAMPLE definition.
 *
 * \if NODOC
 * $Id: imagefilterresample.h_v 1.18 2012/04/05 13:28:15 mju Exp $
 *
 * $Log: imagefilterresample.h_v $
 * Revision 1.18  2012/04/05 13:28:15  mju
 * Use helper_resample.
 *
 * Revision 1.17  2011/10/11 14:31:42  mju
 * Use trans2dmapgen hdr.
 *
 * Revision 1.16  2011/03/31 15:11:51  mju
 * Add v_getTypeName.
 *
 * Revision 1.15  2011/01/06 15:23:07  mju
 * Add getTransToPrev.
 *
 * Revision 1.14  2010/12/01 14:23:41  mju
 * Add m_ksize.
 *
 * Revision 1.13  2010/11/30 15:56:47  mju
 * Add bicubicSmoother/Sharper.
 *
 * Revision 1.12  2009/02/10 14:59:41  mju
 * Add allowEstimate parm to computeSrcMaxSectionSize.
 *
 * Revision 1.11  2008/11/04 18:45:13  mju
 * Add setUsePrevStageZoom and corresponding member.
 *
 * Revision 1.10  2007/03/28 22:29:12  dfriberg
 * Inlined SetAccuracy
 *
 * Revision 1.9  2007/02/14 15:51:19  mju
 * Add ctor using general transformation.
 *
 * Revision 1.8  2007/01/04 17:29:33  mju
 * Add m_pullSinglePixels.
 *
 * Revision 1.7  2006/12/21 18:07:50  mju
 * Add computeSrcMaxSectionSize.
 *
 * Revision 1.6  2006/11/01 23:11:09  mju
 * queryIterator.
 *
 * Revision 1.5  2006/10/16 17:49:42  mju
 * Use imageresamplemethod hdr.
 *
 * Revision 1.4  2006/07/31 14:25:01  mju
 * Add setAccuracy and additional documentation.
 *
 * Revision 1.3  2006/07/28 21:36:37  mju
 * Add pullAffine.
 *
 * Revision 1.2  2006/07/28 20:50:16  mju
 * Add queryStepOrder.
 *
 * Revision 1.1  2006/07/28 17:32:38  mju
 * Initial revision
 *
 * \endif
**/

#ifndef	INC_SPATMOD_IMAGEFILTERRESAMPLE_H
#define	INC_SPATMOD_IMAGEFILTERRESAMPLE_H

#ifndef	INC_SPATMOD_IMAGEFILTER_H
#include <spatmod/imagefilter.h>
#endif

#ifndef	INC_SPATMOD_IMAGESECTION_H
#include <spatmod/imagesection.h>
#endif

#ifndef	INC_SPATMOD_IMAGESAMPLEITERATOR_H
#include <spatmod/imagesampleiterator.h>
#endif

#ifndef	INC_SPATMOD_IMAGEHELPERRESAMPLE_H
#include <spatmod/imagehelperresample.h>
#endif

#ifndef	INC_MI32_TRANS2DMAPGEN_H
#include <mi32/trans2dmapgen.h>
#endif

#ifndef	INC_MI32_RECT_H
#include <mi32/rect.h>
#endif

namespace SPATMOD {
namespace IMAGE {

class BUFFER;

//=====================================================================================================================

//! Filter to resample/reproject image.
//! The coordinate transformation between resulting image and input stage is computed using georeference as
//! specified in constructor, or georeference from specified reference image, along with the georeference
//! from the input stage.  An optional transformation accuracy may be specified via SetAccuracy() to allow use
//! of efficient affine transformation if appropriate.
//! Note that source image will not be processed "sequentially", and the sections specified
//! in PullBegin for the source will overlap.  It is recommended that this filter be placed early in the pipeline or
//! that a suitable caching filter be placed ahead of it if the input requires significant computation or sequential
//! iteration.
class FILTER_RESAMPLE : public FILTER {
	public:

		//! Construct using specified dimensions and georeference.
		FILTER_RESAMPLE (
			STAGE *pPreviousStage,					//!< Pointer to previous stage
			bool TakeOwnership,						//!< Filter takes ownership of previous stage and will delete when needed
			const DIMENSIONS& Dimensions,			//!< Resulting image dimensions
			const GEOREFERENCE& Georeference,	//!< Georeference for resulting image
			RESAMPLEMETHOD ResampleMethod			//!< Resampling method
			);

		//! Construct to match specified reference image.
		FILTER_RESAMPLE (
			STAGE *pPreviousStage,					//!< Pointer to previous stage
			bool TakeOwnership,						//!< Filter takes ownership of previous stage and will delete when needed
			const BASE& RefImage,					//!< Reference image to match
			RESAMPLEMETHOD ResampleMethod			//!< Resampling method
			);

		//! Construct using specified dimensions and general transformation.
		//! Note, the forward direction of the specified transformation is from the resulting image to the previous image.
		//! Use of previous stage zoom support is not available with this constructor.
		FILTER_RESAMPLE (
			STAGE *pPreviousStage,					//!< Pointer to previous stage
			bool TakeOwnership,						//!< Filter takes ownership of previous stage and will delete when needed
			const DIMENSIONS& Dimensions,			//!< Resulting image dimensions
			const TRANS2D_MAPGEN& TransToPrev,	//!< Transformation from resulting image to previous image
			RESAMPLEMETHOD ResampleMethod			//!< Resampling method
			);

		// Destructor.
		virtual ~FILTER_RESAMPLE ();

		//! Get transformation to previous stage.
		//! Care must be taken if uses previous stage zoom support as that will not be included in transformation.
		const TRANS2D_MAPGEN& GetTransToPrev () const
			{ return (m_TransToPrev); }

		//! Set transformation accuracy in source cells.
		//! If within the specified accuracy, a fast affine transformation will be used for computation.
		//! If used, must be done before Initialize().
		//! Default accuracy is .5 cells.
		void SetAccuracy (
			double accuracy							//!< Transformation accuracy in source cells.
			) { m_Accuracy = accuracy; }

		//! Set whether to use previous stage zoom support if possible.
		//! Default is to use unless general transformation constructor was used.
		//! If used, must be done before Initialize().
		void SetUsePrevStageZoom (
			bool UsePrevStageZoom
			) { m_UsePrevStageZoom = UsePrevStageZoom; }

	private:
		#ifndef GENERATING_DOXYGEN_OUTPUT
		RESAMPLEMETHOD m_ResampleMethod;
		TRANS2D_MAPGEN m_TransToPrev;
		DRECT2D m_SrcRectTotal;
		DOUBLE_ARRAY<DPOINT2D> m_SrcCoord;
		DOUBLE_ARRAY<DPOINT2D> m_TgtCoord;
		HELPER_RESAMPLE m_Resampler;
		double m_Accuracy;
		bool m_PullSinglePixels;
		bool m_HasTrans;
		bool m_UsePrevStageZoom;

		ERRVALUE ComputeSrcMaxSectionSize (
			const SECTION_ITERATOR& TgtIterator,
			DRECT2D& SrcRectAll,
			UINT32& SrcMaxColumns,
			UINT32& SrcMaxRows,
			bool AllowEstimate
			) const;

		ERRVALUE PullGeneral (const SECTION& SubSection, BUFFER& buffer);

		// BASE/STAGE/FILTER overrides.
		virtual const char* v_GetTypeName () const;
		virtual ERRVALUE v_Initialize ();
		virtual ERRVALUE v_PullBegin (const SECTION_ITERATOR& iterator);
		virtual ERRVALUE v_PullEnd ();
		virtual ERRVALUE v_PullSubSection (const SECTION& SubSection, BUFFER& buffer);
		virtual float v_QueryIterator (const SECTION_ITERATOR& iterator) const;

		// Unimplemented
		FILTER_RESAMPLE (const FILTER_RESAMPLE&);
		FILTER_RESAMPLE& operator= (const FILTER_RESAMPLE&);
		#endif // GENERATING_DOXYGEN_OUTPUT

	};

//=====================================================================================================================

}	// End namespace IMAGE
}	// End namespace SPATMOD

#endif	// INC_SPATMOD_IMAGEFILTERRESAMPLE_H

