/**
 * \file spatmod/imagefilterstatistics.h
 * \brief Spatial Modelling (SPATMOD) IMAGE::FILTER_STATISTICS definition.
 *
 * \if NODOC
 * $Id: imagefilterstatistics.h_v 1.2 2010/09/27 15:32:32 mju Exp $
 *
 * $Log: imagefilterstatistics.h_v $
 * Revision 1.2  2010/09/27 15:32:32  mju
 * Add value_None.
 *
 * Revision 1.1  2010/09/27 14:21:22  mju
 * Initial revision
 *
 * \endif
**/

#ifndef	INC_SPATMOD_IMAGEFILTERSTATISTICS_H
#define	INC_SPATMOD_IMAGEFILTERSTATISTICS_H

#ifndef	INC_SPATMOD_IMAGEFILTER_H
#include <spatmod/imagefilter.h>
#endif

#ifndef	INC_SPATMOD_IMAGESAMPLEITERATOR_H
#include <spatmod/imagesampleiterator.h>
#endif

#ifndef GENERATING_DOXYGEN_OUTPUT
class MISTRINGLIST;
#endif

namespace SPATMOD {
namespace IMAGE {

#ifndef GENERATING_DOXYGEN_OUTPUT
class SAMPLEITERATOR;
class SAMPLEITERATOR_CONST;
#endif

//=====================================================================================================================

//! Filter to compute various statistics for each pixel.
//! Output pixels will be "valid" if any input sample is valid.  If only a single input sample is valid, output
//! for StdDev, Variance and RegressionSlope will be 0.
class FILTER_STATISTICS : public FILTER {
	public:

		//!< Values which may be computed.
		enum VALUES {
			VALUE_None =					0,
			VALUE_Count =					0x0001,		//!< Number of valid component values
			VALUE_Diversity =				0x0002,		//!< Number of different component values
			VALUE_Minimum =				0x0004,		//!< Minimum value
			VALUE_Maximum =				0x0008,		//!< Maximum value
			VALUE_Median =					0x0010,		//!< Median value
			VALUE_Mode =					0x0020,		//!< Value occuring most frequently
			VALUE_Mean =					0x0040,		//!< Mean of values
			VALUE_Sum =						0x0080,		//!< Sum of values
			VALUE_StdDev =					0x0100,		//!< Standard deviation
			VALUE_Variance =				0x0200,		//!< Variance
			VALUE_RegressionSlope =		0x0400,		//!< Regression line slope, input sample number is X
			VALUE_RegressionOffset =	0x0800,		//!< Regression line y-intercept with input sample number as X
			VALUE_All =						0x0FFF,
			VALUE_LAST =					0x0800
			};

		//! Get TEXTID for value name.
		static TEXTID GetValueName (
			VALUES value
			);

		//! Construct with automatic datatype selection.
		//! Data types will be chosen such that overflow cannot occur.
		//! If no input has DATATYPE_FLOAT64, then DATATYPE_FLOAT32 will be used for Mean, Sum, StdDev, Variance and Regression.
		FILTER_STATISTICS (
			STAGE_ARRAY& PreviousStages,
			bool TakeOwnership,
			VALUES values							//!< Values to output
			);

		//! Construct with specific output datatypes.
		FILTER_STATISTICS (
			STAGE_ARRAY& PreviousStages,
			bool TakeOwnership,
			VALUES values,							//!< Values to output
			DATATYPE DataTypeMinMaxMedMode,	//!< Data type for Minimum, Maximum, Median and Mode, DATATYPE_Invalid to determine automatically
			DATATYPE DataTypeMean,				//!< Data type for Mean, DATATYPE_Invalid to determine automatically
			DATATYPE DataTypeSum,				//!< Data type for Sum, DATATYPE_Invalid to determine automatically
			DATATYPE DataTypeStdVarReg			//!< Data type for StdDev, Variance and Regression, DATATYPE_Invalid to determine automatically
			);

		virtual ~FILTER_STATISTICS ();

	private:
		#ifndef GENERATING_DOXYGEN_OUTPUT
		typedef FILTER BASECLASS;

		VALUES m_Values;
		DATATYPE m_DataTypeMinMaxMedMode;
		DATATYPE m_DataTypeMean;
		DATATYPE m_DataTypeSum;
		DATATYPE m_DataTypeStdVarReg;
		BUFFER *m_BufSrc;
		SAMPLEITERATOR_CONST *m_itSampleSrc;
		SAMPLEITERATOR *m_itSampleTgt;

		// STAGE/FILTER overrides.
		virtual ERRVALUE v_Initialize ();
		virtual ERRVALUE v_PullSubSection (const SECTION& SubSection, BUFFER& buffer);

		#endif
	};

#ifndef GENERATING_DOXYGEN_OUTPUT
DEFINE_ENUM_OP_BITWISE(FILTER_STATISTICS::VALUES)
#endif

//=====================================================================================================================

}	// End namespace IMAGE
}	// End namespace SPATMOD

#endif	// INC_SPATMOD_IMAGEFILTERSTATISTICS_H

