/**
 * \file spatmod/imageformat.h
 * \brief SPATMOD::IMAGE::FORMAT enumeration
 *
 * \if NODOC
 * $Id: imageformat.h_v 1.12 2012/05/07 16:44:18 bcolombe Exp $
 *
 * $Log: imageformat.h_v $
 * Revision 1.12  2012/05/07 16:44:18  bcolombe
 * Add kml
 *
 * Revision 1.11  2012/04/03 20:53:01  bcolombe
 * Changes for new terrain format
 *
 * Revision 1.10  2012/03/14 21:59:35  bcolombe
 * Add 24,32 bit terrain
 *
 * Revision 1.9  2012/02/20 17:55:56  bcolombe
 * Support new png 16 format
 *
 * Revision 1.8  2012/01/19 21:31:25  bcolombe
 * Add PNG16
 *
 * Revision 1.7  2012/01/09 20:16:36  bcolombe
 * Add dae
 *
 * Revision 1.6  2011/08/25 19:32:31  ldyas
 * adding svg support
 *
 * Revision 1.5  2010/06/03 15:19:13  bcolombe
 * Added GIF
 *
 * Revision 1.4  2009/08/13 16:45:30  mju
 * Add factory_target_file and ctor to use.
 *
 * Revision 1.3  2009/08/06 13:30:46  mju
 * Add getSpec.
 *
 * Revision 1.2  2009/07/09 17:09:08  mju
 * Change to contain spec/extn.
 *
 * Revision 1.1  2009/07/09 16:32:49  mju
 * Initial revision
 *
 * \endif
**/

#ifndef	INC_SPATMOD_IMAGEFORMAT_H
#define	INC_SPATMOD_IMAGEFORMAT_H

namespace SPATMOD {
namespace IMAGE {

class FACTORY_TARGET_FILE;

//! Container for image format specification and file extension.
class FORMAT {
	public:

		//! Enumeration for common format specifications.
		//! Each generally corresponds to a specific set of file extensions and SOURCE/TARGET pair.
		enum SPEC {
			SPEC_Undefined,
			SPEC_JPEG,
			SPEC_PNG,
			SPEC_TIFF,
			SPEC_JP2,
			SPEC_GIF,
			SPEC_SVG,
			SPEC_SVGZ,
			SPEC_DAE,
			SPEC_PNG16,
			SPEC_PNG16A,
			SPEC_PNG16OLD,
			SPEC_PNG24,
			SPEC_PNG32,
			SPEC_KML
			};

		//! Default constructor for undefined specification.
		FORMAT ();

		//! Construct with given specification.
		FORMAT (
			FORMAT::SPEC spec,
			const char *FileExtn = 0			//!< Optional file extension, will use default 3-character extension if 0
			);

		//! Construct with TARGET_FILE::FACTORY
		FORMAT (
			const FACTORY_TARGET_FILE& FactoryTgt,			
			const char *FileExtn = 0			//!< Optional file extension, will use default 3-character extension if 0
			);

		//! Construct from string.
		FORMAT (
			const char *str,						//!< String in MIME format (eg "image/jpeg");
			const char *FileExtn = 0			//!< Optional file extension, will use default 3-character extension if 0
			);

		//! Copy constructor.
		FORMAT (const FORMAT& rhs);

		//! Destructor.
		~FORMAT ();

		//! Assignment.
		FORMAT& operator= (const FORMAT& rhs);

		//! Get file extension specified when constructed or default if none specified.
		const char* GetFileExtn (
			) const;

		//! Get factory for constructing TARGET_FILE instance if defined.
		//! @return NULL if no factory set.
		const FACTORY_TARGET_FILE* GetFactoryTgt (
			) const { return (m_pFactoryTgt); }

		//! Get MIME type string (eg "image/jpeg") for specification.
		//! @return 0 if none defined.
		const char* GetMimeType (
			) const;

		//! Get specification enumeration.
		SPEC GetSpec (
			) const { return (m_Spec); }

	private:
		#ifndef GENERATING_DOXYGEN_OUTPUT
		FORMAT::SPEC m_Spec;
		char *m_FileExtn;
		FACTORY_TARGET_FILE *m_pFactoryTgt;
		#endif
	};

}	// End namespace SPATMOD
}	// End namespace IMAGE

#endif	// INC_SPATMOD_IMAGESPEC_H
