/**
 * \file imagepixeltable.h <spatmod/imagepixeltable.h>
 * \brief Spatial Modelling (SPATMOD) IMAGE::PIXELTABLE class definition
 *
 * \if NODOC
 * $Id: imagepixeltable.h_v 1.3 2007/01/02 22:23:51 mju Exp $
 *
 * $Log: imagepixeltable.h_v $
 * Revision 1.3  2007/01/02 22:23:51  mju
 * Redesign to use array of sample tables as is much more efficient.
 *
 * Revision 1.2  2005/12/15 18:08:25  mju
 * Add getColors.
 *
 * Revision 1.1  2005/08/16 15:10:31  mju
 * Initial revision
 *
 * \endif
**/

#ifndef	INC_SPATMOD_IMAGEPIXELTABLE_H
#define	INC_SPATMOD_IMAGEPIXELTABLE_H

#ifndef	INC_SPATMOD_IMAGEPIXEL_H
#include <spatmod/imagepixel.h>
#endif

#ifndef	INC_SPATMOD_IMAGESAMPLETABLE_H
#include <spatmod/imagesampletable.h>
#endif

struct COLOR;
class COLOR_ARRAY;

namespace SPATMOD {
namespace IMAGE {

//=====================================================================================================================

//! Image pixel table.
class PIXELTABLE {
	public:

		//! Default constructor.
		PIXELTABLE ();

		//! Copy constructor
		PIXELTABLE (
			const PIXELTABLE& rhs
			);

		//! Destructor.
		~PIXELTABLE ();

		//! Create pixel table.
		//! Will also set all items to value specified by properties.
		ERRVALUE Create (
			const PIXEL& PixelProperties,		//!< Pixel properties
			UINT32 NumItems						//!< Number of table items
			);

		//! Create from COLOR array.
		//! If any color in array uses transparency the pixel type will be TYPE_RGBA,
		//! otherwise the pixel type will be TYPE_RGB.  The data type will be DATATYPE_UINT8.
		ERRVALUE Create (
			const COLOR *colors,					//!< Array of colors
			INT32 NumItems							//!< Number of array entries
			);

		//! Exchange data with another PIXELTABLE.
		void Exchange (
			PIXELTABLE& other						//!< Pixel table to exchange with
			);

		//! Get colors as array.
		//! Error will be returned if cannot convert.
		ERRVALUE GetColors (
			COLOR_ARRAY& Colors					//! Array of colors returned
			) const;

		//! Get item from table.
		ERRVALUE GetItem (
			UINT32 index,							//!< Index in table
			PIXEL& value							//!< Pixel value returned
			) const;

		//! Get number of items in table.
		UINT32 GetNumItems (
			) const;

		//! Get pixel properties.
		const PIXEL& GetPixelProperties (
			) const { return (m_PixelProperties); }

		//! Get pointer to single item.
		//! For efficiency should not be used in pixel processing loops.
		const void* GetPtrItem (
			UINT16 SampleIdx,
			UINT32 ItemIndex
			) const { return (GetSampleTable(SampleIdx).GetPtrItem(ItemIndex)); }

		//! Get reference to sample table.
		const SAMPLETABLE& GetSampleTable (
			UINT16 SampleIdx						//!< Sample index
			) const { return (m_SampleTables[SampleIdx]); }

		//! Get reference to sample table (non-const).
		SAMPLETABLE& GetSampleTable (
			UINT16 SampleIdx						//!< Sample index
			) { return (m_SampleTables[SampleIdx]); }

		//! Set all items to same value.
		ERRVALUE SetAllItems (
			const PIXEL& value					//!< Pixel value to set
			);

		//! Set table item with index and property validation.
		ERRVALUE SetItem (
			UINT32 index,							//!< Index in table
			const PIXEL& value					//!< Pixel value to set
			);

	private:
		#ifndef GENERATING_DOXYGEN_OUTPUT
		PIXEL m_PixelProperties;
		SAMPLETABLE *m_SampleTables;

		PIXELTABLE& operator= (const PIXELTABLE& rhs);
		#endif // GENERATING_DOXYGEN_OUTPUT
	};

//=====================================================================================================================

}	// End namespace IMAGE
}	// End namespace SPATMOD

#endif	 // INC_SPATMOD_IMAGEPIXELTABLE_H
