/**
 * \file imagesourcememory.h <spatmod/imagesourcememory.h>
 * \brief Spatial Modelling (SPATMOD) IMAGE::SOURCE_MEMORY definition.
 *
 * \if NODOC
 * $Id: imagesourcememory.h_v 1.6 2009/08/05 16:57:28 mju Exp $
 *
 * $Log: imagesourcememory.h_v $
 * Revision 1.6  2009/08/05 16:57:28  mju
 * Add getBuffer.
 *
 * Revision 1.5  2009/07/07 13:24:42  mju
 * Add pixeltable parm when create from buffer.
 *
 * Revision 1.4  2009/01/28 21:51:12  mju
 * Use const for buffer in ctor and attach.
 *
 * Revision 1.3  2009/01/28 21:40:01  mju
 * Add ctor from buffer.
 *
 * Revision 1.2  2006/11/01 23:10:10  mju
 * queryIterator.
 *
 * Revision 1.1  2006/07/27 21:49:16  mju
 * Initial revision
 *
 * \endif
**/

#ifndef	INC_SPATMOD_IMAGESOURCEMEMORY_H
#define	INC_SPATMOD_IMAGESOURCEMEMORY_H

#ifndef	INC_SPATMOD_IMAGESOURCE_H
#include <spatmod/imagesource.h>
#endif

#ifndef	INC_SPATMOD_IMAGEBUFFER_H
#include <spatmod/imagebuffer.h>
#endif

namespace SPATMOD {
namespace IMAGE {

//=====================================================================================================================

//! Source which supports reading from memory.
class SOURCE_MEMORY : public SOURCE {
	public:

		//! Construct using reference image for dimensions and pixel properties.
		//! After construction must call either AttachBuffer or CopyFromBuffer to fill in the image.
		explicit SOURCE_MEMORY (
			const IMAGE::BASE& RefImage				//!< Reference image for dimensions and properties
			);

		//! Construct using buffer.
		explicit SOURCE_MEMORY (
			const IMAGE::BUFFER& buffer,				//!< Buffer to attach and determine properties from
			const IMAGE::PIXELTABLE *pPixelTable = 0	//!< Pixel lookup table for if "indexed"
			);

		// Destructor.
		virtual ~SOURCE_MEMORY ();

		//! Attach buffer to image.
		//! Must be done before initialize.
		//! Buffer must have same dimensions and pixel properties as this image.
		ERRVALUE AttachBuffer (
			const IMAGE::BUFFER& buffer				//!< Buffer to attach
			);

		//! Copy from buffer to image.
		//! Must be done before initialize.
		//! Buffer must have same dimensions and pixel properties as this image.
		ERRVALUE CopyFromBuffer (
			const IMAGE::BUFFER& buffer				//!< Buffer to copy from
			);

		//! Get buffer to use directly.
		const IMAGE::BUFFER& GetBuffer (
			) const { return (m_Buffer); }

	private:
		#ifndef GENERATING_DOXYGEN_OUTPUT
		typedef SOURCE BASECLASS;

		IMAGE::BUFFER m_Buffer;

		// BASE overrides.
		virtual float v_QueryIterator (const SECTION_ITERATOR& iterator) const;

		// STAGE overrides.
		virtual ERRVALUE v_Initialize ();
		virtual ERRVALUE v_PullBegin (const SECTION_ITERATOR& iterator);
		virtual ERRVALUE v_PullSubSection (const SECTION& SubSection, BUFFER& buffer);

		// Unimplemented
		SOURCE_MEMORY (const SOURCE_MEMORY&);
		SOURCE_MEMORY& operator= (const SOURCE_MEMORY&);
		#endif // GENERATING_DOXYGEN_OUTPUT

	};

//=====================================================================================================================

}	// End namespace IMAGE
}	// End namespace SPATMOD

#endif	// INC_SPATMOD_IMAGESOURCEMEMORY_H

