/**
 * \file spatmod/imagetargetrvcmultifile.h
 * \brief Spatial Modelling (SPATMOD) IMAGE::TARGET_RVC_MULTIFILE definition.
 *
 * \if NODOC
 * $Id: imagetargetrvcmultifile.h_v 1.16 2008/11/11 15:07:03 mju Exp $
 *
 * $Log: imagetargetrvcmultifile.h_v $
 * Revision 1.16  2008/11/11 15:07:03  mju
 * Support multiband output.
 *
 * Revision 1.15  2007/08/27 20:00:55  mju
 * Add delegates for tile begin/end so can keep track of completed steps.
 *
 * Revision 1.14  2007/08/16 17:05:40  mju
 * Add setTileNamePrefix.
 *
 * Revision 1.13  2007/06/25 14:58:28  mju
 * Add m_objMask.
 *
 * Revision 1.12  2007/06/22 15:02:56  mju
 * Add setActualDimensions.
 *
 * Revision 1.11  2007/06/21 13:27:38  mju
 * J2K parms no longer nested.
 *
 * Revision 1.10  2007/05/24 13:54:50  mju
 * Add ability to set parms for tiff.
 *
 * Revision 1.9  2007/05/23 18:01:18  mju
 * Add more structure types.
 * Add ability to set base folder path.
 *
 * Revision 1.8  2007/04/18 15:53:13  dfriberg
 * Remove unneeded members
 *
 * Revision 1.7  2007/02/28 22:06:26  dfriberg
 * New directory / file naming scheme.
 *
 * Revision 1.6  2007/02/13 01:06:19  dfriberg
 * Add directory structure enum.
 *
 * Revision 1.4  2007/01/26 23:27:44  dfriberg
 * Add virtual methods to be called when writing one output file per tile
 *
 * Revision 1.3  2007/01/17 21:53:29  dfriberg
 * Add QueryIterator virtual method
 *
 * Revision 1.2  2006/12/13 22:13:01  dfriberg
 * Set up to create and populate multiraster
 *
 * Revision 1.1  2006/12/11 14:35:01  mju
 * Initial revision
 *
 * \endif
**/

#ifndef	INC_SPATMOD_IMAGETARGETRVCMULTIFILE_H
#define	INC_SPATMOD_IMAGETARGETRVCMULTIFILE_H

#ifndef	INC_SPATMOD_IMAGETARGETRVCBASE_H
#include <spatmod/imagetargetrvcbase.h>
#endif

#ifndef	INC_SPATMOD_IMAGETARGETFILE_H
#include <spatmod/imagetargetfile.h>
#endif

#ifndef INC_RVC_MULTRAST_H
#include <rvc/multrast.h>
#endif

#ifndef INC_RVC_NULLMASK_H
#include <rvc/nullmask.h>
#endif


namespace SPATMOD {
namespace IMAGE {

class TARGETPARM_ECW;
class TARGETPARM_J2K;
class TARGETPARM_MRSID;
class TARGETPARM_TIFF;

//=====================================================================================================================

//! RVC multi-raster (tiled) target.
class TARGET_RVC_MULTIFILE : public TARGET_RVC_BASE {
	public:

		//! Format of external tiles.
		//! Values are chosen to match RVC::RASTER::LINKTYPE values, be sure additions match also
		enum FORMAT {
			FORMAT_TIFF =		2,			//!< TIFF file
			FORMAT_MrSID =		4,			//!< LizardTech MrSID file
			FORMAT_ECW =		5,			//!< ERMapper ECW file
			FORMAT_JP2 =		6,			//!< JPEG2000 JP2 file
			FORMAT_PNG =		8,			//!< PNG file
			FORMAT_JPEG =		9,			//!< JPEG file
			};

		//! Directory structure to use when storing tile files.
		enum STRUCTURE {
			STRUCTURE_General_SingleFolder,				//!< All tiles in one folder for each resolution level
			STRUCTURE_General_TreeTileRow,				//!< Tiles in folders by row, folder name is row index
			STRUCTURE_General_TreeTileColumn,			//!< Tiles in folders by column, folder name is column index
//			STRUCTURE_General_Tree2D,
			STRUCTURE_OSGEO_TileMapService,				//!< OSGEO Tile Map Service structure and naming
//			STRUCTURE_Google_Maps,							//!< Google Maps structure and naming
//			STRUCTURE_Google_Earth,							//!< Google Earth structure and naming
//			STRUCTURE_NASA_WorldWind,						//!< NASA WorldWind structure
//			STRUCTURE_Microsoft_VirtualEarth,			//!< Microsoft Virtual Earth
			};

		enum PROFILE {
			PROFILE_None,
			PROFILE_Global_Geodetic,
			PROFILE_Global_Mercator,
//			PROFILE_Local
			};

		//! Delegate type for notifying when tile to be added.
		typedef fastdelegate::FastDelegate<ERRVALUE(INT32, INT32, const FILEPATH&, const LRECT2D&)> DELEGATE_TILE_ADD;

		//! Construct target with single output object.
		//! If source image cannot be written to single object an error will occur when Initialize() is called.
		TARGET_RVC_MULTIFILE (
			STAGE *pInputStage,								//!< Stage in pipeline to obtain image data from
			FORMAT format,										//!< Image tile file format
			const RVC::OBJITEM& ObjItem,					//!< Object location and descriptor
			INT32 ResLevel,
			INT32 ColumnsPerTile,
			INT32 RowsPerTile,
			const RVC::CREDENTIALS *pCredentials = 0	//!< Optional credentials
			);

		//! Construct target with multiple output objects.
		//! If source image cannot be written to multiple objects an error will occur when Initialize() is called.
		TARGET_RVC_MULTIFILE (
			STAGE *pInputStage,								//!< Stage in pipeline to obtain image data from
			FORMAT format,										//!< Image tile file format
			const RVC::OBJITEMLIST& ObjItemList,		//!< Object locations and descriptors
			INT32 ResLevel,
			INT32 ColumnsPerTile,
			INT32 RowsPerTile,
			const RVC::CREDENTIALS *pCredentials = 0	//!< Optional credentials
			);

		//! Destructor.
		virtual ~TARGET_RVC_MULTIFILE ();

		//! Set actual dimensions of RVC raster itself.
		//! This allows for exclusion of part of last column/row of tiles.
		//! Must be done before initialize.  If not used, will match input stage dimensions.
		ERRVALUE SetActualDimensions (
			INT32 ActualColumns,
			INT32 ActualRows
			);

		//! Set delegate to call when new tile is to be added.
		//! Will not be called for entirely empty tiles.
		//! Can be used to keep track of when new step has begun.
		void SetDelegateOnTileAdd (
			DELEGATE_TILE_ADD delegate
			) { m_DelegateOnTileAdd = delegate; }

		//! Set delegate to call at end of tile.
		//! Will not be called for entirely empty tiles.
		//! Can be used to keep track of when step has completed.
		void SetDelegateOnTileEnd (
			DELEGATE_ERRVALUE_NOPARMS delegate
			) { m_DelegateOnTileEnd = delegate; }

		void SetOutputStructure (
			STRUCTURE structure
			) { m_Structure = structure; }

		//! Set external georeference formats to save with each tile.
		//! Default is to not save georeference in file separate from tile.
		void SetGeoFormats (
			TARGET_FILE::GEOFORMATS GeoFormats
			) { m_GeoFormats = GeoFormats; }

		//! Set parameters specific to JP2 format.
		void SetParmsJP2 (
			const TARGETPARM_J2K& parm
			);

		//! Set parameters specific to JPEG format.
		void SetParmsJPEG (
			INT16 quality
			) { m_QualityJPEG = quality; }

		//! Set parameters specific to TIFF format.
		void SetParmsTIFF (
			const TARGETPARM_TIFF& Parm
			);

		void SetProfile (
			PROFILE profile
			) { m_Profile = profile; }
		
		//! Set tile folder base path.
		//! If not set will be derived from RVC file and raster name.
		void SetTileFolderBasePath (
			const FILEPATH& filepath
			) { m_BaseFilePath = filepath; }

		//! Set prefix for tile file names.
		void SetTileNamePrefix (
			const MISTRING& NamePrefix
			) { m_TileNamePrefix = NamePrefix; }

	private:
		#ifndef GENERATING_DOXYGEN_OUTPUT

		typedef TARGET_RVC_BASE BASECLASS;

		STRUCTURE m_Structure;
		PROFILE m_Profile;
		FORMAT m_format;
		INT32 m_ActualColumns;
		INT32 m_ActualRows;
		INT32 m_ResLevel;
		INT32 m_ColumnsPerTile;
		INT32 m_RowsPerTile;
		int m_ColumnFmtDigits;
		int m_RowFmtDigits;
		bool m_IsComposite;
		TARGET_FILE::GEOFORMATS m_GeoFormats;
		TARGET_FILE *m_pTargetFile;
		TARGETPARM_J2K *m_pSettingsJP2;
		TARGETPARM_TIFF *m_pParmsTIFF;
		UINT16 m_QualityJPEG;
		FILEPATH m_BaseFilePath;
		FILEPATH m_TileFilePath;
		MISTRING m_TileNamePrefix;
		LRECT2D m_TileExtents;
		RVC::MULTIRASTER *m_MultiRasters;
		RVC::NULLMASK m_ObjMask;
		DELEGATE_TILE_ADD m_DelegateOnTileAdd;
		DELEGATE_ERRVALUE_NOPARMS m_DelegateOnTileEnd;

		ERRVALUE AddTileLink (
			UINT16 SampleIdx = 0
			);

		ERRVALUE OnTileAdd (
			INT32 TileColumn,									// Tile column
			INT32 TileRow,										// Tile row
			FILEPATH& filepath,								// Path to tile file returned
			const LRECT2D& extents							// Extents of tile in overall image coordinates
			);

		ERRVALUE OnTileEnd (
			);

		// TARGET overrides.
		virtual ERRVALUE v_Initialize ();
		virtual ERRVALUE v_PushBegin (const SECTION_ITERATOR& iterator);
		virtual ERRVALUE v_PushEnd ();
		virtual ERRVALUE v_PushSubSection (const SECTION& SubSection, const BUFFER& buffer);
		virtual float v_QueryIterator ( const SECTION_ITERATOR& iterator) const;

		// Unimplemented
		TARGET_RVC_MULTIFILE (const TARGET_RVC_MULTIFILE&);
		TARGET_RVC_MULTIFILE& operator= (const TARGET_RVC_MULTIFILE&);
		#endif // GENERATING_DOXYGEN_OUTPUT

	};

//=====================================================================================================================

}	// End namespace IMAGE
}	// End namespace SPATMOD

#endif	// INC_SPATMOD_IMAGETARGETRVCMULTIFILE_H

