/**
 * \file imagetargetjpeg.h <spatmod/imagetargetjpeg.h>
 * \brief Spatial Modelling (SPATMOD) IMAGE::TARGET_TIFF definition.
 *
 * \if NODOC
 *
 * $Id: imagetargettiff.h_v 1.7 2009/08/13 16:46:37 mju Exp $
 *
 * $Log: imagetargettiff.h_v $
 * Revision 1.7  2009/08/13 16:46:37  mju
 * Base factory not nested.
 *
 * Revision 1.6  2009/08/13 15:38:29  mju
 * Add factory.
 *
 * Revision 1.5  2007/08/20 14:26:34  mju
 * Remove illegal qualifier.
 *
 * Revision 1.4  2007/05/24 13:55:06  mju
 * Parms moved to separate class so can set in target_rvc_multifile.
 *
 * Revision 1.3  2007/04/12 16:28:56  dfriberg
 * Enable multifile output
 *
 * Revision 1.2  2007/01/26 23:27:53  dfriberg
 * Add virtual methods to be called when writing one output file per tile
 *
 * Revision 1.1  2005/12/16 16:09:15  mju
 * Initial revision
 *
 * \endif
**/

#ifndef	INC_SPATMOD_IMAGETARGETTIFF_H
#define	INC_SPATMOD_IMAGETARGETTIFF_H

#ifndef	INC_SPATMOD_IMAGETARGETFILE_H
#include <spatmod/imagetargetfile.h>
#endif

#ifndef	INC_MI32_TIFFEXT_H
#include <mi32/tiffext.h>
#endif

class TIFFEXT;

namespace SPATMOD {
namespace IMAGE {

//=====================================================================================================================

//! Settings for TIFF image target.
class TARGETPARM_TIFF {
	public:

		//! Enumeration for type of GeoTIFF tags to use.
		enum GEOTAG {
			GEOTAG_None,
			GEOTAG_Automatic,
			GEOTAG_Affine,
			GEOTAG_CtrlPoint
			};

		//! Constructor.
		TARGETPARM_TIFF (
			);

		//! Set compression type.
		//! Default is COMPRESSION_None if not specified.
		void SetCompression (
			TIFFEXT::COMPRESSION compression			//!< Compression method
			) { m_Compression = compression; }

		//! Set image description.
		//! Default is no description if not specified.
		void SetDescription (
			const MISTRING& desc							//!< Description
			) { m_Description = desc; }

		//! Set preferred type of GeoTIFF tags to use.
		//! Default is GEOTAG_Automatic if not specified.
		void SetGeoTag (
			GEOTAG GeoTag
			) { m_GeoTag = GeoTag; }

		//! Set planar configuration.
		//! Default is PLANARCONFIG_Contiguous if not specified.
		void SetPlanarConfig (
			TIFFEXT::PLANARCONFIG PlanarConfig
			) { m_PlanarConfig = PlanarConfig; }

		//! Set segment size.
		//! Actual segment size may be adjusted based on format requirements.
		//! Default segment size will be determined from input stage if not specified.
		void SetSegmentSize (
			INT32 SegmentColumns,						//!< Number of columns per segment
			INT32 SegmentRows								//!< Number of rows per segment
			) { m_SegmentColumns = SegmentColumns; m_SegmentRows = SegmentRows; }

	private:
		#ifndef GENERATING_DOXYGEN_OUTPUT
		MISTRING m_Description;
		TIFFEXT::COMPRESSION m_Compression;
		TIFFEXT::PLANARCONFIG m_PlanarConfig;
		INT32 m_SegmentColumns;
		INT32 m_SegmentRows;
		GEOTAG m_GeoTag;
		friend class TARGET_TIFF;
		#endif // GENERATING_DOXYGEN_OUTPUT
	};

//=====================================================================================================================

//! TIFF image target.
class TARGET_TIFF : public TARGET_FILE {
	public:

		//! Factory class for constructing TARGET instance.
		class FACTORY : public FACTORY_TARGET_FILE {
			public:

				//! Default constructor.
				FACTORY ();

				//! Construct with TARGETPARM_TIFF.
				explicit FACTORY (	
					const TARGETPARM_TIFF& parm
					);

				//! Copy constructor.
				FACTORY (const FACTORY& rhs);

				//! Destructor.
				virtual ~FACTORY ();

			private:
				#ifndef GENERATING_DOXYGEN_OUTPUT
				TARGETPARM_TIFF m_Parm;

				// FACTORY_TARGET_FILE overrides.
				virtual FACTORY_TARGET_FILE * v_Clone () const;
				virtual TARGET_FILE * v_ConstructTarget (STAGE *pInputStage, const FILEPATH& filepath, GEOFORMATS GeoFormats) const;

				// Unimplemented.
				FACTORY& operator= (const FACTORY& rhs);
				#endif
			};

		//! Construct target with single output object.
		//! If source image cannot be written to single object an error will occur when Initialize() is
		//! called, use constructor with OBJITEMLIST if multiple objects are to be created.
		TARGET_TIFF (
			STAGE *pInputStage,							//!< Stage in pipeline to obtain image data from
			const FILEPATH& filepath,					//!< Path to target file
			GEOFORMATS GeoFormats = GEOFORMAT_None	//!< External georeference format
			);

		//! Destructor.
		virtual ~TARGET_TIFF ();

		//! Set optional parameters.
		//! If used, must be called before Initialize().
		void SetParms (
			const TARGETPARM_TIFF& parms
			);

	private:
		#ifndef GENERATING_DOXYGEN_OUTPUT
		TARGETPARM_TIFF m_Parm;
		TIFFEXT_FILE *m_pFile;
		SIMPLE_ARRAY<UINT8> m_RowBuf;

		// Private methods
		ERRVALUE MakeFile ( const FILEPATH& Filepath, const LRECT2D& Extents);

		// TARGET overrides
		virtual ERRVALUE v_FillTileRows ( INT32 NumColumns, INT32 NumRows);
		virtual ERRVALUE v_Initialize ();
		virtual ERRVALUE v_PushBegin (const SECTION_ITERATOR& iterator);
		virtual ERRVALUE v_PushEnd ();
		virtual ERRVALUE v_TileBegin (const	FILEPATH& filepath,const LRECT2D& extents);
		virtual ERRVALUE v_TileEnd ();
		virtual ERRVALUE v_WriteSubSection (const SECTION& SubSection, const BUFFER& buffer);

		// Unimplemented
		TARGET_TIFF (const TARGET_TIFF&);
		TARGET_TIFF& operator= (const TARGET_TIFF&);
		#endif // GENERATING_DOXYGEN_OUTPUT

	};

//=====================================================================================================================

}	// End namespace IMAGE
}	// End namespace SPATMOD

#endif	// INC_SPATMOD_IMAGETARGETTIFF_H

