/**
 * \file spatmod/imagetargettileset.h
 * \brief Spatial Modelling (SPATMOD) IMAGE::TARGET_TILESET definition.
 *
 * \if NODOC
 * $Id: imagetargettileset.h_v 1.12 2012/10/25 22:57:16 bcolombe Exp $
 *
 * $Log: imagetargettileset.h_v $
 * Revision 1.12  2012/10/25 22:57:16  bcolombe
 * Add copyright
 *
 * Revision 1.11  2012/01/26 22:31:07  bcolombe
 * Add IsTerrain
 *
 * Revision 1.10  2011/08/03 16:02:22  bcolombe
 * Add method to set tile ranges when known
 *
 * Revision 1.9  2011/04/22 19:30:18  bcolombe
 * Add IsOnBorder when computing tile values
 *
 * Revision 1.8  2011/04/20 23:15:45  bcolombe
 * Add ResampleMethod
 *
 * Revision 1.7  2010/11/01 23:02:34  bcolombe
 * Imgformat changes
 *
 * Revision 1.6  2010/10/18 18:57:59  bcolombe
 * Add SetMetadata
 *
 * Revision 1.5  2009/08/13 17:25:32  bcolombe
 * Add SetLevelRange that takes extents
 *
 * Revision 1.4  2009/08/13 16:46:43  mju
 * Use factory.
 *
 * Revision 1.3  2009/08/06 17:01:19  mju
 * *** empty log message ***
 *
 * Revision 1.2  2009/08/05 18:55:10  mju
 * Adjust docs for delegate to say is relative.
 *
 * Revision 1.1  2009/08/05 17:42:49  mju
 * Initial revision
 *
 * \endif
**/

#ifndef	INC_SPATMOD_IMAGETARGETTILESET_H
#define	INC_SPATMOD_IMAGETARGETTILESET_H

#ifndef	INC_SPATMOD_IMAGETARGET_H
#include <spatmod/imagetarget.h>
#endif

#ifndef	INC_SPATMOD_IMAGEBUFFER_H
#include <spatmod/imagebuffer.h>
#endif

#ifndef	INC_SPATMOD_IMAGETILESET_H
#include <spatmod/imagetileset.h>
#endif

#ifndef INC_SPATMOD_IMAGERESAMPLEMETHOD_H
#include <spatmod/imageresamplemethod.h>
#endif

namespace SPATMOD {
namespace IMAGE {

class SOURCE_MEMORY;

//=====================================================================================================================

//! Base for tileset targets.
class TARGET_TILESET : public TARGET {
	public:
		//! Delegate type for notifying when tile to be added.
		//! Parameters are tile column, row relative in output image (not necessarily same as global tileset).
		typedef fastdelegate::FastDelegate<ERRVALUE(INT32, INT32)> DELEGATE_COLUMN_ROW;

		//! Destructor.
		virtual ~TARGET_TILESET ();

		//! Get ImgFormat
		TILESET::IMGFORMAT GetImgFormat (
			) const { return (m_format); }

		//! Get resample method to use when computing reduced levels
		SPATMOD::IMAGE::RESAMPLEMETHOD GetResampleMethod (
			) const { return (m_ResampleMethod); }

		//! Is tileset terrain version
		bool IsTerrain (
			) const {return m_bIsTerrain;}

		//! Set delegate to call when new tile has been added.
		//! Will not be called for entirely empty tiles.
		//! Can be used to keep track of steps completed for resuming incomplete job.
		void SetDelegateOnTileAdded (
			DELEGATE_COLUMN_ROW delegate
			) { m_DelegateOnTileAdded = delegate; }

		//! Set ImgFormat
		void SetImgFormat (
			TILESET::IMGFORMAT ImgFormat
			) { m_format = ImgFormat; }

		//! Set level range, must be called before Initialize() for non-local profile.
		//! For local profile, level range is set automatically.
		ERRVALUE SetLevelRange (
			INT32 MinLevel,
			INT32 MaxLevel
			);

		//! Set level range, must be called before Initialize() for non-local profile.
		//! For local profile, level range is set automatically.
		ERRVALUE SetLevelRange (
			const DRECT2D& ExtentsInCRS,				//!< Extents must be in CRS of tileset
			INT32 MinLevel,
			INT32 MaxLevel,
			bool IsOnBorder	= false						//!< True if extents boundary lies on borders between tiles
			);

		//! Set tile range for max level.  Will compute range for all levels.
		ERRVALUE SetMaxTileRange (
			const TILESET::RANGE& TileRange		//!< Tile Range at max level
			);

		//! Set tileset's copyright
		void SetCopyright (
			const MISTRING& copyright
			) { m_tileset.SetCopyright(copyright); }

		//! Set tileset's metadata
		void SetMetadata (
			const MISTRING& metadata
			) { m_tileset.SetMetadata(metadata); }

		//! Set resample method to use when computing reduced levels
		void SetResampleMethod (
			SPATMOD::IMAGE::RESAMPLEMETHOD ResampleMethod
			) { m_ResampleMethod = ResampleMethod; }

		//! Set to create terrain version of tileset (using pseudo 16-bit PNG tiles)
		void SetIsTerrain (
			bool bIsTerrain
			) { m_bIsTerrain = bIsTerrain; }

	protected:

		//! Construct with TILESET specification.
		TARGET_TILESET (
			STAGE *pInputStage,							//!< Stage in pipeline to obtain image data from
			const IMAGE::TILESET& tileset				//!< Complete tileset specification
			);

		//! Construct with tilesize, profile and format.
		TARGET_TILESET (
			STAGE *pInputStage,							//!< Stage in pipeline to obtain image data from
			INT32 TileSize,								//!< Tile Size
			IMAGE::TILESET::PROFILE profile,			//!< Tileset profile
			const IMAGE::FORMAT format					//!< Image tile file format
			);

		//! Construct with tilesize and profile.
		TARGET_TILESET (
			STAGE *pInputStage,							//!< Stage in pipeline to obtain image data from
			INT32 TileSize,								//!< Tile Size
			IMAGE::TILESET::PROFILE profile			//!< Tileset profile
			);

		//! Get TILESET definition.
		const TILESET& GetTileset (
			) const { return (m_tileset); }

		//! Get TILESET definition (non-const).
		TILESET& GetTileset (
			) { return (m_tileset); }

		//! Initialize - derived class must call base.
		virtual ERRVALUE v_Initialize ();

	private:
		#ifndef GENERATING_DOXYGEN_OUTPUT

		IMAGE::TILESET m_tileset;
		IMAGE::TILESET::IMGFORMAT m_format;
		LRECT2D m_CurTileExtents;
		bool m_bIsTerrain;
		bool m_DidTileBegin;
		BUFFER m_TileBuffer;
		IMAGE::RESAMPLEMETHOD m_ResampleMethod;
		DELEGATE_COLUMN_ROW m_DelegateOnTileAdded;

		void Construct (IMAGE::TILESET::PROFILE profile, INT32 TileSize);

		// TARGET overrides.
		virtual ERRVALUE v_PushSubSection (const SECTION& SubSection, const BUFFER& buffer);
		virtual float v_QueryIterator (const SECTION_ITERATOR& iterator) const;

		// Unimplemented
		TARGET_TILESET (const TARGET_TILESET&);
		TARGET_TILESET& operator= (const TARGET_TILESET&);
		#endif // GENERATING_DOXYGEN_OUTPUT

		// Overridables.

		//! Write tile.
		virtual ERRVALUE v_WriteTile (
			SOURCE_MEMORY& SourceTile,					//!< Source for accessing data to write tile
			const LPOINT3D& TileIdx,					//!< Tile index (x = column, y = row, z = zoom level)
			bool HasNullCells								//!< True if tile has any null cells
			) = 0;
	};

//=====================================================================================================================

}	// End namespace IMAGE
}	// End namespace SPATMOD

#endif	// INC_SPATMOD_IMAGETARGETTILESET_H

